#define DEBUG

#ifdef DEBUG
#include <stdio.h>
#endif

#include "pic24_all.h"


_CONFIG2(IESO_OFF & FNOSC_FRCDIV & FCKSM_CSECMD & OSCIOFNC_ON & IOL1WAY_OFF  & POSCMOD_NONE)
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF & WINDIS_OFF & FWPSA_PR32 & WDTPS_PS1)

#define FOSC_HZ 		    FCY
#define BEEP_DISCR_THRESHOLD   30


typedef unsigned long ul;
typedef signed long   sl;

#define SENSOR_DFL_WINDOW_SECS           2
#define SENSOR_MIN_WINDOW_SECS           2
#define SENSOR_MAX_WINDOW_SECS           5

#define NUM_PZLUTS      1
#define PZLUT_ENTRIES   56
#define PA_INIT         102860
#define PA_DELTA        1024

// maximum oversampling
#define SENSOR_OSS					3

// temperature sample takes 4.5mS, pressure takes 25.5mS

#define SENSOR_TEMP_ACQ_MS			 5
#define SENSOR_PRESS_ACQ_MS			26

// 5mS temperature sample + 26mS pressure sample = 31mS total per sample 
// which is about 32 samples/second
#define SENSOR_SAMPLES_PER_SEC        32
#define SNS_NUM_Z_SAMPLES            160



volatile s16 gnTempCx10;
int gnKLPFC;
int gnWindowSecs;
int gnWindowSamples;
int gnPZTblSel;
int gnKLPFC;
int gnWindowSecs;
int gnWindowSamples;
int gnPZTblSel = 0;

volatile int gnAcqTicks;
volatile s32 gZBuf[SNS_NUM_Z_SAMPLES];
volatile int gnSmpCnt;
volatile s32 gnAvgAltM; 
volatile s32 gnAltCm;
volatile s32 gnCps;
volatile s32 gnTraw;
volatile s32 gnPraw;
volatile s32 gnCpsx32;
volatile int gnTempC;
volatile u16 gnAcqState = 0;

volatile s16 gnBeepPeriodTicks = 0;
volatile s16 gnBeepTransition;
volatile s16 gnBeepCps = 0;
volatile s16 gnVarioCps = 0;

char gszBuf[80];
int n;
volatile s32 gnFreqHz;



void mcu_Init(void);


void sns_Init(void);
void sns_ReadSample(void);
void sns_InitZBuf(void);


s32  var_Pa2Cm(s32 pa);
void var_ClimbRate();
void var_Beep(void);
void var_SetFreq(s32 gnFreqHz);

#define I2C_BITBANG

#ifdef I2C_BITBANG

#define I2CERR_NAK          11
int gi2c_error;

#define SCL			_LATB8		// pin 17

#define SDA			_RB9		// pin 18
#define	SDA_TRIS	_TRISB9	

void i2c_Init(void);
u8   i2c_RcvByte(u8 devId, u8 addr);
int  i2c_XmtByte(u8 devId, u8 addr, u8 data);
int  i2c_RcvBuf(u8 devId, u8 addr, int nBytes, u8* pBuf);
int  i2c_XmtBuf(u8 devId, u8 addr, int nBytes, u8* pBuf );
u8   i2c_RByte(int ack);
int  i2c_XByte(u8 dat);
int  i2c_RBit(void);
int  i2c_XBit(int bit);
int  i2c_Stop(void);
int  i2c_Start(void);
#endif

u8 gnLCDX = 0;
u8 gnLCDY = 0;



#define gpioLcdClk		    	_RB15		// pin 26
#define gpioLcdDta		    	_RB14		// pin 25
#define gpioLcdDc	    	    _RB13		// pin 24
#define gpioLcdRst	    		_RB12		// pin 23

#define gpioLED                 _RB6
#define gpioSnsXclr	    		_RB6		// pin 16
#define gpioSnsEoc	    		_RB7		// pin 15

void lcd_Init(void);				
void lcd_WriteCmd(char cmd);			
void lcd_WriteDat(char dat);			
void lcd_ClockData(char dat);				
void lcd_ClearScreen(void);			
void lcd_SetPos(char y,char x);			
void lcd_PutChar(char ch);					
void lcd_PrintSz(char * sz);			
void lcd_PutCharX2 (char ch);
void lcd_PrintSzX2(char *sz);

void  tmr1_config(void);
void _ISRFAST _T1Interrupt (void);
void adc_Config(void);

void tmr3_BusyWaitMs(int mS);
int  tmr3_ElapsedTimeMs(void);
void tmr3_Start(void);
void tmr3_Config(void);
void tmr2_Config(void);

char gnFontTbl[310] = {	
                     0x00,0x00,0x00,0x00,0x00,	// 20 space		
							0x00,0x00,0x5f,0x00,0x00,	// 21 !			
							0x00,0x07,0x00,0x07,0x00,	// 22 "			
							0x14,0x7f,0x14,0x7f,0x14,	// 23 #			
							0x39,0x44,0x42,0x42,0x26,	// 24 $	Celsius		
							0x23,0x13,0x08,0x64,0x62,	// 25 %
							0x36,0x49,0x55,0x22,0x50,	// 26 &
							0x00,0x05,0x03,0x00,0x00,	// 27 '
							0x00,0x1c,0x22,0x41,0x00,	// 28 (
							0x00,0x41,0x22,0x1c,0x00,	// 29 )
							0x14,0x08,0x3e,0x08,0x14,	// 2a *
							0x08,0x08,0x3e,0x08,0x08,	// 2b +
							0x07,0x81,0xb7,0xD1,0x17,	// 2c ,   m/s
							0x08,0x08,0x08,0x08,0x08,	// 2d -
							0x00,0x60,0x60,0x00,0x00,	// 2e .
							0x20,0x10,0x08,0x04,0x02,	// 2f /
							0x3e,0x51,0x49,0x45,0x3e,	// 30 0
							0x00,0x42,0x7f,0x40,0x00,	// 31 1
							0x42,0x61,0x51,0x49,0x46,	// 32 2
							0x21,0x41,0x45,0x4b,0x31,	// 33 3
							0x18,0x14,0x12,0x7f,0x10,	// 34 4
							0x27,0x45,0x45,0x45,0x39,	// 35 5
							0x3c,0x4a,0x49,0x49,0x30,	// 36 6
							0x01,0x71,0x09,0x05,0x03,	// 37 7
							0x36,0x49,0x49,0x49,0x36,	// 38 8
							0x06,0x49,0x49,0x29,0x1e,	// 39 9
							0x00,0x36,0x36,0x00,0x00,	// 3a :
							0x00,0x56,0x36,0x00,0x00,	// 3b ;
							0x08,0x14,0x22,0x41,0x00,	// 3c <
							0x14,0x14,0x14,0x14,0x14,	// 3d =
							0x00,0x41,0x22,0x14,0x08,	// 3e >
							0x02,0x01,0x51,0x09,0x06,	// 3f ?
							0x32,0x49,0x79,0x41,0x3e,	// 40 @
							0x20,0x54,0x54,0x54,0x78,	// 41 a
							0x7f,0x48,0x44,0x44,0x38,	// 42 b
							0x38,0x44,0x44,0x44,0x20,	// 43 c
							0x38,0x44,0x44,0x48,0x7f,	// 44 d
							0x38,0x54,0x54,0x54,0x18,	// 45 e
							0x08,0x7e,0x09,0x01,0x02,	// 46 f
							0x0c,0x52,0x52,0x52,0x3e,	// 47 g
							0x7f,0x08,0x04,0x04,0x78,	// 48 h
							0x00,0x44,0x7d,0x40,0x00,	// 49 i
							0x20,0x40,0x44,0x3d,0x00,	// 4a j 
							0x7f,0x10,0x28,0x44,0x00,	// 4b k
							0x00,0x41,0x7f,0x40,0x00,	// 4c l
							0x7c,0x04,0x18,0x04,0x78,	// 4d m
							0x7c,0x08,0x04,0x04,0x78,	// 4e n
							0x38,0x44,0x44,0x44,0x38,	// 4f o
							0x7c,0x14,0x14,0x14,0x08,	// 50 p
							0x08,0x14,0x14,0x18,0x7c,	// 51 q
							0x7c,0x08,0x04,0x04,0x08,	// 52 r
							0x48,0x54,0x54,0x54,0x20,	// 53 s
							0x04,0x3f,0x44,0x40,0x20,	// 54 t
							0x3c,0x40,0x40,0x20,0x7c,	// 55 u
							0x1c,0x20,0x40,0x20,0x1c,	// 56 v
							0x3c,0x40,0x30,0x40,0x3c,	// 57 w
							0x44,0x28,0x10,0x28,0x44,	// 58 x
							0x0c,0x50,0x50,0x50,0x3c,	// 59 y
							0x44,0x64,0x54,0x4c,0x44,	// 5a z
							0x7f,0x7f,0x3e,0x1c,0x08,	// 5b [   selected
							0x02,0x04,0x08,0x10,0x20,	// 5c Yen 
							0x7f,0x81,0x81,0x81,0x7f};	// 5d ]   

#ifdef DEBUG
void uart_Init(void);
void uart_Print(char *szBuf);
void dbg_PrintAltitude(s16 alt);
void dbg_PrintTemperature(s16 temp);
void dbg_PrintClimbRate(s16 cps);
#endif // DEBUG

void sns_LPFClimbRate(void);
void sns_TriggerPressureSample(void);
void sns_TriggerTemperatureSample(void);
s32  sns_ReadPressureSample(void);
s32  sns_ReadTemperatureSample(void);
s32  sns_CalcTemperatureCx10(void);
s32  sns_CalcPressurePa(void);
void sns_DeltaCmPerSec(void);
s32  sns_Pa2Cm(s32 pa, int pzInx);
void sns_AverageAltitude(void);
void sns_Sample_ISR(void);
void sns_Init(void);
void sns_InitData(void);
void sns_InitWindowBuffers(void);
void sns_ReadCoeffs(void);
void sns_Reset(void);
void sns_StopSensorAcqTmr(void);
void sns_StartSensorAcqTmr(void);
void sns_AcquireAveragedSample(int nSamples, int pzInx);

#define I2C_ID_DS1338            ((u8)0xD0)
#define I2C_ID_BMP085            ((u8)0xEE)

#ifdef I2C_HW
char I2Cread(char addr, char subaddr);
void I2Cwrite(char addr, char subaddr, char value);
char i2c_read_ack(void); //does not reset bus!!!
char i2c_read(void);
char send_i2c_byte(int data);
void reset_i2c_bus(void);
void i2c_restart(void);
void i2c_start(void);
void i2c_init(int BRG);
#endif

static int gAC1;
static int gAC2;
static int gAC3;
static int gAC4;
static int gAC5;
static int gAC6;
static int gB1;
static int gB2;
static int gB5;
static int gMb;
static int gMc;
static int gMd;

int main(void)
	{
	u8 xmt,rcv;
    int volt;
    mcu_Init();
    lcd_Init();
    tmr3_BusyWaitMs(3000);
    sns_Init();
    volt = (int)((convertADC1() * 3020L)>>10);

  	sprintf(gszBuf,"Bat Voltage %d mV\r\n",volt);
    uart_Print(gszBuf);

    xmt = 0;
    gnCps = -400;
    while(gnCps < 1100){
        lcd_SetPos(0,0);
        sprintf(gszBuf,"%04ldcps %04ldHz\r\n",gnCps, gnFreqHz);
        uart_Print(gszBuf);
  		lcd_PrintSz(gszBuf);
       tmr3_BusyWaitMs(1000);
       gnCps += 20;
#ifdef I2C_HW
	//	rcv = I2Cread(0xD0,0x00);
#endif
#ifdef I2C_BITBANG        
       rcv = i2c_RcvByte(0xD0,0x0) & 0x7F;
#endif
       sprintf(gszBuf,"%d ",(int) (((rcv>>4)*10) + (rcv&0xf)));
       uart_Print(gszBuf);
		}

	while(1);

    return 0;
	}


void mcu_Init(void)	{
    //TRISB = 0;
    //_ODB12 = 1;
    AD1PCFG = 0xFFFF;

	CONFIG_U1TX_TO_RP(0); // RP0/pin4/PGD  
	CONFIG_U1RX_TO_RP(1); // RP1/pin5/PGC 
	CONFIG_OC1_TO_RP(3);  // RP3/pin7 (tone generator)

    // gpioLED : digital out
	CONFIG_RB6_AS_DIG_OUTPUT(); // pin 15
    gpioSnsXclr = 1;

	CONFIG_RB7_AS_DIG_OUTPUT(); // pin 16 XCLR
    gpioSnsXclr = 1;

    // gpio lcd  : digital out
	//CONFIG_RB12_AS_DIG_OUTPUT(); // pin 23 RST_
    //DISABLE_RB12_ANALOG();
    //ENABLE_RB12_OPENDRAIN();
    //_TRISB12 = 0;
	CONFIG_RB13_AS_DIG_OUTPUT(); // pin 24 D_C
	CONFIG_RB14_AS_DIG_OUTPUT(); // pin 25 DTA
	CONFIG_RB15_AS_DIG_OUTPUT(); // pin 26 CLK

    //gpioLcdRst = 1;
//	OSCCONbits.IOLOCK = 1;
    CLKDIVbits.RCDIV = 3;  // divide by 8 => FOsc = 500KHz
    tmr1_config();
    tmr2_Config();
    tmr3_Config();
 	adc_Config();
    OC1CONbits.OCM = 5;  // continuous pulse mode
    OC1CONbits.OCTSEL = 0; // use TMR2
#ifdef I2C_BITBANG
    i2c_Init();
#endif
#ifdef I2C_HW
    i2c_init(4);
#endif
#ifdef DEBUG
    uart_Init();
#endif
	}

// TMR2 : 16 bit, vario square wave tone generator
void tmr2_Config(void){
    T2CONbits.T32 = 0; // 16bit timer
    T2CONbits.TCS = 0; //  select internal clock 
    T2CONbits.TGATE = 0; // gate disabled
    T2CONbits.TSIDL = 0; // continue in idle mode
    T2CONbits.TCKPS1 = 0; // Prescaler 1:1
    T2CONbits.TCKPS0 = 0; // Prescaler 1:1
	}

void adc_Config(void) {
  	CONFIG_AN0_AS_ANALOG();
	configADC1_ManualCH0(ADC_CH0_POS_SAMPLEA_AN0, 31);
	}

// TMR3 : 16bit, for delay and code timing functions
void  tmr3_Config(void) {
   T3CONbits.TON = 0;    // turn off the timer
   T3CONbits.TCS = 0;    // select internal clock 
   T3CONbits.TGATE = 0;  // gate disabled
   T3CONbits.TSIDL = 0;  // continue in idle mode
   T3CONbits.TCKPS1 = 1; // Prescaler 1:64
   T3CONbits.TCKPS0 = 0; // Prescaler 1:64

   PR3 = 65535;
   TMR3  = 0;            //clear timer3 value
   _T3IF = 0;            //clear interrupt flag
   _T3IP = 1;            //choose a priority
   _T3IE = 0;            //disable the interrupt
   }


void tmr3_Start(void) {
   	T3CONbits.TON = 0; //turn off the timer
   	TMR3  = 0;         //zero timer3 counter
   	T3CONbits.TON = 1; //turn on the timer
	}

int tmr3_ElapsedTimeMs(void) {
    int ms;
   	T3CONbits.TON = 0; //turn off the timer
    ms = (int)(((TMR3*1000L)<<6)/FOSC_HZ);
    return ms;
	}



// @ 500kHz and prescaler 64, max delay is  8388 mS
void tmr3_BusyWaitMs(int mS) {
    int cnt = (int)(((mS * FOSC_HZ)/1000L)>>6);
   	T3CONbits.TON = 0; //turn off the timer
   	TMR3  = 0;         //zero timer3 counter
   	T3CONbits.TON = 1; //turn on the timer
    while(TMR3 < cnt);
    T3CONbits.TON = 0;
    return;
	}

#define TMR1_PERIOD_MS     25      // in ms

// TMR1 : 16 bit, vario beep ISR timer

void  tmr1_config(void) {
   T1CONbits.TON = 0; //turn off the timer
   T1CONbits.TCS = 0; //  select internal clock 
   T1CONbits.TGATE = 0; // gate disabled
   T1CONbits.TSIDL = 0; // continue in idle mode
   T1CONbits.TCKPS1 = 0; // Prescaler 1:1
   T1CONbits.TCKPS0 = 0; // Prescaler 1:1

   PR1 = (TMR1_PERIOD_MS * FOSC_HZ) / 1000L;
   TMR1  = 0;                       //zero timer1 counter
   _T1IF = 0;                       //clear interrupt flag
   _T1IP = 1;                       //choose a priority
   _T1IE = 1;                       //enable the interrupt
   T1CONbits.TON = 1;               //turn on the timer
   }


void _ISRFAST _T1Interrupt (void) {
  	//gpioLED = !gpioLED; //toggle the LED
  	_T1IF = 0;    //clear the timer interrupt bit
  	var_Beep();
	}


void var_SetFreq(s32 gnFreqHz){
    u16 tmrTicks;
    T2CONbits.TON = 0; // stop tmr2
    if (gnFreqHz) {
    	tmrTicks = (u16)(FOSC_HZ/gnFreqHz);
    	OC1RS = tmrTicks;
    	OC1R = (tmrTicks >> 1); // square wave
    	PR2 = (tmrTicks+1);
		}
    else { // dc 0V
        OC1RS = 100;
        OC1R = 50;
        PR2 = 40;
        }
    T2CONbits.TON = 1; // start tmr2
    }

    






#define VARIO_MAX_CPS   	1000L
#define VARIO_XOVER_CPS		500L

#define VARIO_MAX_FREQHZ    5000L
#define VARIO_XOVER_FREQHZ  2000L
#define VARIO_MIN_FREQHZ    200L

#define VARIO_SINK_CPS   	-300L
#define VARIO_SINK_FREQHZ   1300L


u8 gnBeepTbl[20][2] = {
{17,13}, // 0.0 - 0.5
{16,13}, // 0.5 - 1.0
{15,12}, // 1.0 - 1.5
{14,12}, // 1.5 - 2.0
{13,11}, // 2.0 - 2.5
{12,11}, // 2.5 - 3.0
{11,9}, // 3.0 - 3.5
{10,9}, // 3.5 - 4.0
{9,7}, // 4.0 - 4.5
{8,6}, // 4.5 - 5.0
{7,6}, // 5.0 - 5.5
{7,6}, // 5.5 - 6.0
{6,5}, // 6.0 - 6.5
{6,5}, // 6.5 - 7.0
{5,4}, // 7.0 - 7.5
{5,4}, // 7.5 - 8.0
{5,4}, // 8.0 - 8.5
{5,4}, // 8.5 - 9.0
{5,4}, // 9.0 - 9.5
{5,4} // 9.5 - 10.0
};

void var_Beep(void)
	{
    int nRange;

   	if ((gnBeepPeriodTicks <= 0) || (ABS(gnCps-gnVarioCps) > BEEP_DISCR_THRESHOLD) || ((gnCps > 0) && (gnVarioCps <= 0)) || ((gnCps <= 0) && (gnVarioCps > 0))) {
   		gnVarioCps = gnCps;
        if (gnVarioCps > VARIO_MAX_CPS) gnVarioCps = VARIO_MAX_CPS;
      	if (gnVarioCps < VARIO_SINK_CPS) {
			if (gnBeepCps >= VARIO_SINK_CPS) {
				gnBeepCps = gnVarioCps;	
				gnFreqHz = VARIO_SINK_FREQHZ;
            	var_SetFreq(gnFreqHz);
         		gnBeepPeriodTicks = 0;
         		gnBeepTransition  = 0;
				}
         	}
      	else{    
           	gnBeepCps = gnVarioCps;
         	if (gnBeepCps > 0) {
                nRange = gnBeepCps/50;
                if (nRange > 19) nRange = 19;
                gnBeepPeriodTicks = (int)(gnBeepTbl[nRange][0]);
				gnBeepTransition = (int)(gnBeepTbl[nRange][1]);
         		if (gnBeepCps > VARIO_XOVER_CPS){
            		gnFreqHz = VARIO_XOVER_FREQHZ+ ((gnBeepCps - VARIO_XOVER_CPS)*(VARIO_MAX_FREQHZ - VARIO_XOVER_FREQHZ))/(VARIO_MAX_CPS - VARIO_XOVER_CPS);
            		}
         		else{
            		gnFreqHz = VARIO_MIN_FREQHZ + (gnBeepCps*(VARIO_XOVER_FREQHZ - VARIO_MIN_FREQHZ))/VARIO_XOVER_CPS;
					}
            	var_SetFreq(gnFreqHz);
          		}  	
         	else{
    	     	gnBeepPeriodTicks = 0;
	         	gnBeepTransition  = 0;
				gnFreqHz = 0;
               	var_SetFreq(gnFreqHz);
            	}
         	}
   		}
   	else{
      	gnBeepPeriodTicks--;
      	if (gnBeepPeriodTicks <= gnBeepTransition){
            //gnFreqHz = 0;
         	var_SetFreq(0);
         	}
      	}       	
   }

#ifdef I2C_BITBANG

#define I2C_CLK_STRETCH() {Nop();Nop();Nop();Nop();Nop();Nop();Nop();}


void i2c_Init(void)
	{
    CONFIG_RB8_AS_DIG_OUTPUT();
    CONFIG_RB9_AS_DIG_OD_OUTPUT();
    LATBbits.LATB9 = 0;
    I2C_CLK_STRETCH();
    SDA_TRIS = 1;
    SCL = 1;
    I2C_CLK_STRETCH();
    gi2c_error = 0;
	}


int i2c_Start(void)
    {
    SCL = 1; //  SCL is high
    I2C_CLK_STRETCH();
    SDA_TRIS = 0;
    //I2C_CLK_STRETCH();
    //SDA = 0;  // Pull SDA down while SCL is high ... start condition
    I2C_CLK_STRETCH();
    SCL = 0; //  Pull SCL low for next bit
    I2C_CLK_STRETCH();
    return 0;
    }

int i2c_Stop(void)
    {
    SDA_TRIS = 0;
    //I2C_CLK_STRETCH();
    //SDA = 0;  // Pull SDA low
    I2C_CLK_STRETCH();
    SCL = 1;   // Let SCL go up
    I2C_CLK_STRETCH();
    //SDA = 1;  // then SDA
    SDA_TRIS = 1;  // then SDA
    I2C_CLK_STRETCH();
    return 0;
    }

int i2c_XBit(int bit)
   {
//   SDA_TRIS = 0;
//   I2C_CLK_STRETCH();
//   SDA = (bit ? 1 : 0);
   SDA_TRIS = (bit ? 1 : 0);
   I2C_CLK_STRETCH();
   SCL = 1;
   I2C_CLK_STRETCH();
   SCL = 0;
   I2C_CLK_STRETCH();
   return 0;
   }

int i2c_RBit(void)
   {
   int bit;
   SDA_TRIS = 1;
   I2C_CLK_STRETCH();
   SCL = 1;
   I2C_CLK_STRETCH();
   bit = (SDA ? 1 : 0);  
   SCL = 0;
   I2C_CLK_STRETCH();
   return bit;
   }

int i2c_XByte(u8 dat)
    {
    int bitCount = 8;
    while(bitCount--)
        {
        i2c_XBit((dat & 0x80)? 1 : 0);
        dat <<= 1;
        }

    if (i2c_RBit())
        {
        gi2c_error = I2CERR_NAK;
        return 1;
        }
    return 0;
    }

u8 i2c_RByte(int ack)
    {
    u8 inByte = 0;
    int bitCount = 8;

    while(bitCount--)
        {
        inByte <<= 1;
        inByte |= i2c_RBit();
        }

    // acknowledge
    i2c_XBit(ack);
    return inByte;
    }


int i2c_XmtBuf(u8 devId, u8 addr, int nBytes, u8* pBuf )
    {
    int cnt;

    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        return 0xF1;
        }
    for (cnt = 0; cnt < nBytes; cnt++)
        if (i2c_XByte(pBuf[cnt]))
            {
            return 0xF2;
            }
    i2c_Stop();
    return 0;
    }




int i2c_RcvBuf(u8 devId, u8 addr, int nBytes, u8* pBuf)
    {
    int cnt;

    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        if (gi2c_error == I2CERR_NAK) i2c_Stop();
        return 0xF1;
        }
    SDA_TRIS = 0;
    I2C_CLK_STRETCH();
//    SDA = 1; // Set SDA high while SCL is low
    SDA_TRIS = 1; // Set SDA high while SCL is low
    // Restart
    I2C_CLK_STRETCH();
    i2c_Start();
    if (i2c_XByte(devId|0x1))
        {
        return 0xF2;
        }

    for (cnt = 0; cnt < nBytes-1; cnt++)
        {
        pBuf[cnt] = i2c_RByte(0);
        }
    // Master NAK
    pBuf[nBytes-1] = i2c_RByte(1);
    i2c_Stop();
    return (gi2c_error ? 1 : 0);
    }



int i2c_XmtByte(u8 devId, u8 addr, u8 data)
    {
    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        return 0xF1;
        }
    if (i2c_XByte(data))
        {
        return 0xF2;
        }
    i2c_Stop();
    return 0;
    }


u8 i2c_RcvByte(u8 devId, u8 addr)
    {
    u8 data;
    
    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        if (gi2c_error == I2CERR_NAK) i2c_Stop();
        return 0xF1;
        }
    SDA_TRIS = 0;
    I2C_CLK_STRETCH();
//    SDA = 1; // Set SDA high while SCL is low
    SDA_TRIS = 1; // Set SDA high while SCL is low
    I2C_CLK_STRETCH();
    // Restart
    i2c_Start();
    if (i2c_XByte(devId|0x1))
        {
        return 0xF2;
        }
    data = i2c_RByte(1);
    i2c_Stop();
    return data;
    }

#endif


#ifdef DEBUG

void uart_Init(void)
	{
    char tmp;
    U1BRG  = 3;     // 31250baud @ Fosc = 500Khz 
	U1MODEbits.BRGH = 1;
    U1MODEbits.UARTEN = 1;
    U1MODEbits.RTSMD = 1; // no flow control

    while(U1STAbits.URXDA == 1) {
        tmp = U1RXREG;
        }

    // clear IF flags 
    IFS0bits.U1RXIF = 0;
    IFS0bits.U1TXIF = 0;

    // set priority 
    IPC2bits.U1RXIP = 1;
    IPC3bits.U1TXIP = 1;

    // disable interrupt 
    IEC0bits.U1RXIE = 0;
    IEC0bits.U1TXIE = 0;
    // enable transmit
    U1STAbits.UTXEN = 1;
	}

void uart_Print(char *szBuf)
   	{
   	while(*szBuf) {
      	U1TXREG = *szBuf++;   // transfer data byte to TX reg 
      	while(!U1STAbits.TRMT);  // wait for xmit to complete
      	}	    
   	}

void dbg_PrintAltitude(s16 alt)
	{
    sprintf(gszBuf,"%d m\r\n",alt);
    uart_Print(gszBuf);
	}


void dbg_PrintClimbRate(s16 cps)
	{
    sprintf(gszBuf,"%d cps\r\n",cps);
    uart_Print(gszBuf);
	}

void dbg_PrintTemperature(s16 temp)
	{
    sprintf(gszBuf,"%d C\r\n",temp);
    uart_Print(gszBuf);
	}

#endif // DEBUG

void lcd_Init(void)
   {
   int cnt;
   gpioLcdRst = 1;
   tmr3_BusyWaitMs(30);
   gpioLcdRst = 0;
   cnt = 20;
   while (cnt--);		
   gpioLcdRst = 1;		
   lcd_WriteCmd(0x21);		// Activate Chip and H=1.
   lcd_WriteCmd(0x87);		// Set LCD Voltage to about 7V.
   lcd_WriteCmd(0x13);		// Adjust voltage bias.
   lcd_WriteCmd(0x20);		// Horizontal addressing and H=0.
   lcd_ClearScreen();		// Erase all pixel on the DDRAM.
   lcd_WriteCmd(0x0C);		// Display Normal.
   lcd_ClearScreen();

   lcd_SetPos(0,0);
   lcd_PrintSz("ALTI-VARIO");
   lcd_SetPos(2,0);
   lcd_PrintSz("VERSION 1.00");
   }


void lcd_WriteCmd(char command_in)
   {
	gpioLcdDc = 0;
	lcd_ClockData(command_in);	
   }


void lcd_WriteDat(char data_in)
   {
	gpioLcdDc = 1;
	lcd_ClockData(data_in);		
   }


void lcd_ClockData(char bits_in)
   {
	u8 bitcnt = 8;
   while (bitcnt--)
		{
		gpioLcdClk = 0;				
		gpioLcdDta = ((bits_in&0x80) ? 1 : 0);		// MSb first
        gpioLcdClk = 1;								// Data is clocked on the rising edge
		bits_in=bits_in<<1;						
		}
   }


void lcd_ClearScreen(void)
   {
	s16 cnt = 504;  //84 cols x 6 byte rows
	lcd_SetPos(0,0);										
   while (cnt--) lcd_WriteDat(0);
   }

	
void lcd_SetPos(char y, char x)
   {  
	lcd_WriteCmd(0x80|(x&0x7f));	// X axis
	lcd_WriteCmd(0x40|(y&0x07));	// Y axis
   gnLCDX = x;
   gnLCDY = y;
   }


void lcd_PutChar(char ch)
   {
	s16 inx;
   u8 cnt, col;
	if ((ch < 0x20) || (ch > 0x5d)) return;
	inx = 5*(ch - 32);
   cnt = 5;
	while(cnt--)
	   {
		col = gnFontTbl[inx];
		lcd_WriteDat(col);
		inx++;
	   }
   // 1 pixel horizontal spacing
	lcd_WriteDat(0x00); 
   }


void lcd_PutCharX2 (char ch )
   {
   s16 inx;
   u8 cnt,col,b1,b2;
	if ((ch < 0x20) || (ch > 0x5d)) return;	
	inx = 5*(ch - 32);
   cnt = 5;
   while (cnt--)
      {
      col = gnFontTbl[inx];
      b1 =  (col & 0x01) * 3;
      b1 |= (col & 0x02) * 6;
      b1 |= (col & 0x04) * 12;
      b1 |= (col & 0x08) * 24;
      col >>= 4;
      b2 =  (col & 0x01) * 3;
      b2 |= (col & 0x02) * 6;
      b2 |= (col & 0x04) * 12;
      b2 |= (col & 0x08) * 24;
		lcd_WriteDat(b1);
		lcd_WriteDat(b1);
      lcd_SetPos(gnLCDY+1,gnLCDX);
		lcd_WriteDat(b2);
		lcd_WriteDat(b2);
      lcd_SetPos(gnLCDY-1,gnLCDX+2);
      inx++;
      }
   // 1 pixel horizontal spacing
   lcd_WriteDat(0);
   lcd_SetPos(gnLCDY+1,gnLCDX);
   lcd_WriteDat(0);
   lcd_SetPos(gnLCDY-1,gnLCDX+1);
   }


void lcd_PrintSz(char *sz)
   {
	while(*sz) lcd_PutChar(*sz++);								
   }


void lcd_PrintSzX2(char *sz)
   {
	while(*sz) lcd_PutCharX2(*sz++);								
   }


u16 convertADC1(void) {
  	SET_SAMP_BIT_ADC1();                    //start sampling
  	WAIT_UNTIL_CONVERSION_COMPLETE_ADC1();  //wait for conversion to finish
  	return(ADC1BUF0);
	}

/** Configures ADC1 to perform 10-/12-bit conversion on a single channel via
 *  CH0.  The sampling is done against the AVDD and AVSS references and
 *  using the internal ADC clock to conserve timer resourses.  User can
 *  initiate this manual sampling+conversion by setting the SAMP bit nin
 *  AD1CON1.  When the DONE bit in AD1CON1 goes high, the conversion is
 *  done.
 *  \param u16_Ch0PositiveMask Mask to select the single analog input to
 *          convert.  See AD1CHS register and associated <i>defines</i>s in
 *          pic24_adc.h
 *  \param u8_autoSampleTime Number of <i>T</i><sub>AD</sub> periods for
 *          sampling the signal.  This value must be 0-31.  If greater than
 *          31, then 31<i>T</i><sub>AD</sub> will be used.
 */
void configADC1_ManualCH0(u16 u16_ch0PositiveMask, u8 u8_autoSampleTime ) {
  	if (u8_autoSampleTime > 31) u8_autoSampleTime=31;
  	AD1CON1bits.ADON = 0;   // turn off ADC (changing setting while ADON is not allowed)
  	AD1CON1 = ADC_CLK_AUTO + ADC_AUTO_SAMPLING_OFF;
  	AD1CON3 = ADC_CONV_CLK_INTERNAL_RC + (u8_autoSampleTime<<8);
  	AD1CON2 = ADC_VREF_AVDD_AVSS;
  	AD1CHS  = ADC_CH0_NEG_SAMPLEA_VREFN + u16_ch0PositiveMask;
  	AD1CON1bits.ADON = 1;   //turn on the ADC
	}




#define SENSOR_KLPFC_H                  7
#define SENSOR_KLPFC_M                 21
#define SENSOR_KLPFC_L                 32

#define SENSOR_DFL_KLPFC                SENSOR_KLPFC_L
#define SENSOR_MAX_KLPFC                32
#define SENSOR_MIN_KLPFC                 1

#define SENSOR_DFL_PZTBL_SEL             0
#define SENSOR_MIN_PZTBL_SEL             0
#define SENSOR_MAX_PZTBL_SEL             0



void sns_Init(void)
   {
   gnSmpCnt = 0;
   gnAvgAltM = 0;
   gnAltCm = 0;
   gnCps = 0;
   gnCpsx32 = 0;
   gnTempC = 0;
   gnKLPFC = SENSOR_DFL_KLPFC;
   gnPZTblSel = SENSOR_DFL_PZTBL_SEL;
   gnWindowSecs = SENSOR_DFL_WINDOW_SECS;
   gnWindowSamples = gnWindowSecs * SENSOR_SAMPLES_PER_SEC;
   sns_Reset();
   sns_ReadCoeffs();
   sns_AcquireAveragedSample(4, gnPZTblSel);
   }

void sns_Reset(void)
	{	
    gpioSnsXclr = 0;
	tmr3_BusyWaitMs(1);
    gpioSnsXclr = 1;
	tmr3_BusyWaitMs(200);
	}



void sns_ReadCoeffs(void)
	{	
	u8 b[22];
	i2c_RcvBuf(I2C_ID_BMP085,0xAA,22,b);

	gAC1 = (s16)((((u16)b[0])<<8) | (u16)b[1]);
	gAC2 = (s16)((((u16)b[2])<<8) | (u16)b[3]);
	gAC3 = (s16)((((u16)b[4])<<8) | (u16)b[5]);
	
	gAC4 = (((u16)b[6])<<8) | (u16)b[7];
	gAC5 = (((u16)b[8])<<8) | (u16)b[9];
	gAC6 = (((u16)b[10])<<8) | (u16)b[11];

	gB1 = (s16)((((u16)b[12])<<8) | (u16)b[13]);
	gB2 = (s16)((((u16)b[14])<<8) | (u16)b[15]);
	gMb = 0;
	gMc = (s16)((((u16)b[18])<<8) | (u16)b[19]);
	gMd = (s16)((((u16)b[20])<<8) | (u16)b[21]);

  	sprintf(gszBuf,"AC %d %d %d %d %d %d\r\n",gAC1, gAC2, gAC3, (int)gAC4, (int)gAC5, (int)gAC6);
    uart_Print(gszBuf);

  	sprintf(gszBuf,"B %d %d\r\n",gB1, gB2);
    uart_Print(gszBuf);

  	sprintf(gszBuf,"M %d %d %d\r\n",gMb, gMc, gMd);
    uart_Print(gszBuf);
/*
    gAC1 = 11118;
    gAC2 = -919;
    gAC3 = -14209;14207
    gAC4 = 33062;
    gAC5 = 25651;
    gAC6 = 16138;
    gB1 = 5498;
    gB2 = 45;
	gMb = 0;
	gMc = -11137;11075
	gMd = 2431;2432
*/	}

void sns_InitData()
   {
   sns_InitWindowBuffers();
   gnCpsx32 = 0;
   gnCps = 0;
   gnSmpCnt = gnWindowSamples-1;

   sns_TriggerTemperatureSample();
   sns_StartSensorAcqTmr();
   }

void sns_StartSensorAcqTmr(void)
	{
	}

void sns_StopSensorAcqTmr(void)
	{
	}

/*
void sns_Sample_ISR(void)
	{
	int p,t;
	u16 tmrVal;
    M_TIM3CNT_TIMER_STOP;		
	gnAcqTicks++;
	if (gnAcqTicks == 1) {
		gnTraw = ReadTemperatureSample();
		TriggerPressureSample();
        tmrVal = TMR_REG_VAL(SENSOR_PRESS_ACQ_MS,64);
        M_TIM3COUNT_SET(tmrVal);
        M_TIM3CNT_TIMER_START;
  	    t = CalcTemperatureCx10();
        gnTempC = (t >= 0?  (t+5)/10 : (t-5)/10);
		p = CalcPressurePa();
  	    gZBuf[gnSmpCnt] = gnAltCm = Pa2Cm(p, gnPZTblSel);
		gnSmpCnt++;
		if (gnSmpCnt == gnWindowSamples) {
			gnSmpCnt = 0;
			}
         AverageAltitude();
         DeltaCmPerSec();
         LPFClimbRate();
		}
	else
	if (gnAcqTicks == 2) {
		gnPraw = ReadPressureSample();
		TriggerTemperatureSample();
		gnAcqTicks = 0;
        tmrVal = TMR_REG_VAL(SENSOR_TEMP_ACQ_MS,64);
        M_TIM3COUNT_SET(tmrVal);
        M_TIM3CNT_TIMER_START;
		}
	}
*/

void sns_InitWindowBuffers()
   {
   int n;
   sns_AcquireAveragedSample(4, gnPZTblSel);
   n = gnWindowSamples;
   while(n--) {
      gZBuf[n]  = gnAltCm;
      }
   }


void sns_TriggerPressureSample(void)
   {
   i2c_XmtByte(I2C_ID_BMP085,0xF4,0x34+(SENSOR_OSS<<6));
   }

void sns_TriggerTemperatureSample(void)
   {
   i2c_XmtByte(I2C_ID_BMP085,0xF4,0x2E);
   }

int gnCnt;

void sns_AcquireAveragedSample(int nSamples, int pzInx)
   {
   s32 pa,tc,pAccum,tAccum,n;
   
   pAccum = 0;
   tAccum = 0;
   n = nSamples;
   while (n--)  {
      sns_TriggerTemperatureSample();
	  tmr3_BusyWaitMs(5);
      gnTraw = sns_ReadTemperatureSample();
	  tc = sns_CalcTemperatureCx10();
	  //gnTraw = 54518;
	  sns_TriggerPressureSample();
	  tmr3_BusyWaitMs(25);
	  gnPraw = sns_ReadPressureSample();
	  //gnPraw = 21657; 
      pa = sns_CalcPressurePa();
  	  //z = Pa2Cm(pa, pzInx);
	  //ham_DrawText(0,gnCnt++,"P %dpa  Z %dcm",p,z);
	  //ham_DrawText(0,gnCnt++,"T %d.%dC",t/10, t%10);
	  pAccum += pa;
	  tAccum += tc;
      }
   tc = tAccum/nSamples;
   gnTempC = (tc >= 0 ?  (tc+5)/10 : (tc-5)/10);
   pa = pAccum/nSamples;
   gnAltCm = sns_Pa2Cm(pa, pzInx);
   gnAvgAltM = (gnAltCm+50L)/100L;
   sprintf(gszBuf,"Sample : %ldm\r\n",gnAvgAltM);
   uart_Print(gszBuf);
   }


s32 sns_ReadTemperatureSample(void)
	{
	u8 b[2];
	u16 w;
    i2c_RcvBuf(I2C_ID_BMP085,0xF6,2,b);
    w = (((u16)b[0])<<8) | (u16)b[1];
	return (s32)w;
	}

s32 sns_ReadPressureSample(void)
	{
	u8 b[3];
	ul dw;
    i2c_RcvBuf(I2C_ID_BMP085,0xF6,3,b);
    dw = (((ul)b[0])<<16) | (((ul)b[1])<<8) | (ul)b[2];
    dw = (dw >> (8-SENSOR_OSS));
    return (s32)dw;
    }


s32 sns_CalcTemperatureCx10(void)
	{
	sl x1,x2,t;
    x1 = (((sl) gnTraw - (sl) gAC6) * (sl) gAC5) >> 15;
    x2 = ((sl) gMc << 11) / (x1 + gMd);
    gB5 = x1 + x2;
    t = ((gB5+8)>>4);
	return (s32)t;
	}

s32 sns_CalcPressurePa(void)
	{
	sl x1,x2,x3,b3,b6,p;
	ul b4,b7;

	b6 = gB5 - 4000;
	x1 = (b6*b6) >> 12;	 	 
	x1 *= gB2;
	x1 >>=11;

	x2 = (gAC2*b6);
	x2 >>=11;
	x3 = x1 +x2;
	b3 = (((((sl)gAC1 )*4 + x3) <<SENSOR_OSS) + 2) >> 2;

	x1 = (gAC3* b6) >> 13;
	x2 = (gB1 * ((b6*b6) >> 12) ) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (gAC4 * (ul) (x3 + 32768)) >> 15;
     
	b7 = ((ul)(gnPraw - b3) * (50000>>SENSOR_OSS));   
	if (b7 < 0x80000000) {
		p = (b7 << 1) / b4;
		}
	else { 
		p = (b7 / b4) << 1;
		}
   
	x1 = p >> 8;
	x1 *= x1;
	x1 = (x1 * 3038) >> 16;
	x2 = (p * -7357) >> 16;
	p += ((x1 + x2 + 3791) >> 4);	// p in Pa  
    return (s32)p;
	}
	


void sns_AverageAltitude(void)
   {
   int cnt;
   s32 avg = 0;
   cnt = gnWindowSamples;
   while (cnt--)  avg += gZBuf[cnt];
   avg /= gnWindowSamples;
   avg = ((avg >= 0) ? (avg+50L)/100L : (avg-50L)/100L); 
   gnAvgAltM = avg;
   return; 
   }


void sns_DeltaCmPerSec(void)
   {
   int n;
   s32 sum_zt,sum_z,sum_t,sum_t2,trel,num,den,tref,z;
   tref = gnSmpCnt; // time origin is the current sample in window
   sum_zt = sum_z = sum_t = sum_t2 = 0;
   n = gnWindowSamples;
   while (n--)  {
      z = gZBuf[n];
      trel = n-tref;
      if (n > tref) trel -= gnWindowSamples;
      sum_z += z;
      sum_t += trel;
      sum_t2 += (trel*trel);
      sum_zt += (trel*z);
      }
   num = (gnWindowSamples*sum_zt) - (sum_z*sum_t);
   den = (gnWindowSamples*sum_t2) - (sum_t*sum_t);
   gnCps = den ? (s32)((num*SENSOR_SAMPLES_PER_SEC)/den) : 0;
   return;
   }



void sns_LPFClimbRate(void)
   {
   s32 tmp;
   tmp = gnCpsx32*(32-gnKLPFC) + (gnCps<<5)*gnKLPFC;
   gnCpsx32 = (tmp >= 0 ? ((tmp + 16)>>5) : ((tmp - 16)>>5));
   gnCps = (gnCpsx32>>5);
   }  


// Set of 100 pressure-altitude LUTs, each computed for a different mean sea level 
// reference pressure, varying from 98825Pa (LUT0) to 103775Pa (LUT99), with a 
// step of 50Pa. LUT50 has the nominal mean sea level reference 
// pressure of 101325Pa specified by the 1976 US Standard Atmosphere model.
// These tables have been computed for a standard mean sea level temperature of 
// 15 degrees Celsius.


s32 gPZTbl[1][PZLUT_ENTRIES] = 
{{ // 50 : Po = 101325
-12700,
-4244,
4279,
12874,
21541,
30281,
39095,
47986,
56953,
66000,
75126,
84335,
93628,
103006,
112472,
122026,
131672,
141410,
151244,
161174,
171204,
181335,
191570,
201911,
212361,
222922,
233597,
244388,
255300,
266334,
277494,
288782,
300204,
311761,
323457,
335297,
347285,
359424,
371719,
384174,
396795,
409586,
422552,
435700,
449033,
462560,
476285,
490216,
504360,
518724,
533316,
548144,
563216,
578543,
594134,
609999
}};



s32 sns_Pa2Cm(s32 pa, int pzInx)
   {
   s32 inx,pa1,pa2,z1,z2,z;
   if (pa > PA_INIT) {
      z = gPZTbl[pzInx][0];
      }
   else {
      inx = (PA_INIT - pa)>>10;      
      if (inx >= PZLUT_ENTRIES) {
         z = gPZTbl[pzInx][PZLUT_ENTRIES-1];
         }
      else {
         pa2 = PA_INIT - (inx<<10);
         pa1 = (inx > 0 ? pa2+PA_DELTA : pa2);
         z1 = (inx > 0 ? gPZTbl[pzInx][inx-1] : gPZTbl[pzInx][0]);
         z2 = gPZTbl[pzInx][inx];
         z = ((pa1-pa)*z2 + (pa-pa2)*z1)>>10;
         }
      }
   return z;
   }

/*
 * $Header$
 */

#ifdef I2C_HW

void i2c_init(int BRG)
{

      int temp;

      I2C2BRG = BRG; // I2CBRG = 194 for 10Mhz OSCI with PPL with 100kHz I2C clock
      I2C2CONbits.I2CEN = 0; // Disable I2C Mode
      I2C2CONbits.DISSLW = 1; // Disable slew rate control
      IFS3bits.MI2C2IF = 0; // Clear Interrupt
      I2C2CONbits.I2CEN = 1; // Enable I2C Mode
      temp = I2C2RCV; // read buffer to clear buffer full
      reset_i2c_bus(); // set bus to idle 

}


void i2c_start(void)
{

      int x = 0;

      I2C2CONbits.ACKDT = 0; //Reset any previous Ack
      x = 1000;
      while (x--);
      I2C1CONbits.SEN = 1; //Initiate Start condition
      Nop();

      //the hardware will automatically clear Start Bit
      //wait for automatic clear before proceding
      while (I2C2CONbits.SEN);

      x = 1000;
      while (x--);

}

void i2c_restart(void)
{

      int x = 0;

      I2C2CONbits.RSEN = 1; //Initiate restart condition
      Nop();

      //the hardware will automatically clear restart bit
      //wait for automatic clear before proceding
      while (I2C2CONbits.RSEN);

      x = 1000;
      while (x--);

}

void reset_i2c_bus(void)
{

      int x = 0;

      //initiate stop bit
      I2C2CONbits.PEN = 1;

      //wait for hardware clear of stop bit
      while (I2C2CONbits.PEN);

      I2C2CONbits.RCEN = 0;
      IFS3bits.MI2C2IF = 0; // Clear Interrupt
      I2C2STATbits.IWCOL = 0;
      I2C2STATbits.BCL = 0;
      x = 1000;
      while (x--);

}

char send_i2c_byte(int data)
{

      int i;

      while (I2C2STATbits.TBF) { }
      IFS3bits.MI2C2IF = 0; // Clear Interrupt
      I2C2TRN  = data; // load the outgoing data byte

      // wait for transmission
      for (i=0; i<16000; i++)
      {
      if (!I2C2STATbits.TRSTAT) break;
      }
      if (i == 16000)
      {
      return(1);
      }

      // Check for NO_ACK from slave, abort if not found
      if (I2C2STATbits.ACKSTAT == 1)
      {
      reset_i2c_bus();
      return(1);

      }
      i = 5000;
      while(i--);
      return(0);

}

char i2c_read(void)
{

      int i = 0;
      char data = 0;

      //set I2C module to receive
      I2C2CONbits.RCEN = 1;

      //if no response, break
      while (!I2C2STATbits.RBF)
      {
      i ++;
      if (i > 2000) break;
      }

      //get data from I2CRCV register
      data = I2C2RCV;

      //return data
      return data;

}


char i2c_read_ack(void) //does not reset bus!!!
{

      int i = 0;
      char data = 0;

      //set I2C module to receive
      I2C2CONbits.RCEN = 1;

      //if no response, break
      while (!I2C2STATbits.RBF)
      {
      i++;
      if (i > 2000) break;
      }

      //get data from I2CRCV register
      data = I2C2RCV;

      //set ACK to high
      I2C2CONbits.ACKEN = 1;

      i = 5000;
      while (i--);
      //wait before exiting

      //return data
      return data;

} 

    void I2Cwrite(char addr, char subaddr, char value)
    {

          i2c_start();
          send_i2c_byte(addr);
          send_i2c_byte(subaddr);
          send_i2c_byte(value);
          reset_i2c_bus();

    }

    char I2Cread(char addr, char subaddr)
    {
		int x;
          char temp;

          i2c_start();
          send_i2c_byte(addr);
          send_i2c_byte(subaddr);
          x = 5000;
          while(x--);

          i2c_restart();
          send_i2c_byte(addr | 0x01);
          temp = i2c_read();

          reset_i2c_bus();
          return temp;

    }
#endif