#define DEBUG
//#define I2C_HW
//#define I2C_SW
//#define I2C_HW2
#define I2C_SW2

#ifdef DEBUG
#include <stdio.h>
#endif

#include <p24fj64ga002.h>

_CONFIG2(IESO_OFF & FNOSC_FRCDIV & FCKSM_CSECMD & OSCIOFNC_ON & IOL1WAY_OFF & I2C1SEL_PRI & POSCMOD_NONE)
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & ICS_PGx1 & FWDTEN_OFF & WINDIS_OFF & FWPSA_PR32 & WDTPS_PS1)

typedef unsigned long  u32;
typedef unsigned short u16;
typedef unsigned char  u8;
typedef signed long    s32;
typedef signed short   s16;
typedef signed char    s8;

#define ABS(x)                 ((x) < 0 ? -(x) : (x))
#define CLAMP(x,min,max)       {if ((x) <= (min)) (x) = (min); else if ((x) >= (max)) (x) = (max);}
#define CORE(x,t)              {if (ABS(x) <= (t)) (x) = 0;}

#define FOSC_HZ 		    500000L

// function IDs as per datasheet
#define FUNC_UART1_TX 	3
#define FUNC_OC1 		18

// pressure -> altitude lookup table
#define PZLUT_ENTRIES          75
#define PA_INIT                102860L
#define PA_DELTA               1024L

#define BEEP_DISCR_THRESHOLD   30

// about 67-68mS per {Pressure,Temperature} sample
#define SNS_ACQ_PER_SEC         15
#define SNS_NUM_Z_SAMPLES       32

#define gpioMSCLK  	PORTBbits.RB6
#define gpioMDOUT  	PORTBbits.RB7
#define gpioMDIN   	PORTBbits.RB8

#define VARIO_MINFREQ_HZ 	300L
#define VARIO_MAXFREQ_HZ 	5000L
#define VARIO_MAXCPS   		1000L

#define VARIO_SINKTONE_CPS   -350



u16 gwD1;
u16 gwD2;
u16 gwC1;
u16 gwC2;
u16 gwC3;
u16 gwC4;
u16 gwC5;
u16 gwC6;

s32 gP;
s16 gT;

s32 gZBuf[SNS_NUM_Z_SAMPLES];
s16 gnSmpCnt;
s32 gnAltCm;
s16 gnCps;
s16 gnTempCx10;

u16 gnAcqState = 0;

s16 gnBeepPeriodTicks = 0;
s16 gnBeepTransition;
s16 gnBeepCps = 0;
s16 gnVarioCps = 0;

char gszBuf[80];
int n;

s32 gnPZTbl[PZLUT_ENTRIES] = 
{//  Po = 101325
-12700,
-4244,
4279,
12874,
21541,
30281,
39095,
47986,
56953,
66000,
75126,
84335,
93628,
103006,
112472,
122026,
131672,
141410,
151244,
161174,
171204,
181335,
191570,
201911,
212361,
222922,
233597,
244388,
255300,
266334,
277494,
288782,
300204,
311761,
323457,
335297,
347285,
359424,
371719,
384174,
396795,
409586,
422552,
435700,
449033,
462560,
476285,
490216,
504360,
518724,
533316,
548144,
563216,
578543,
594134,
609999,
626149,
642595,
659352,
676431,
693847,
711615,
729752,
748275,
767202,
786555,
806356,
826627,
847395,
868688,
890537,
912974,
936037,
959766,
984206
};

void mcu_Init(void);
void mcu_Delay(void);

void sns_Init(void);
void sns_Reset(void);
u16  sns_ReadCoeff(u8 addr);
u16  sns_ReadWord(void);
void sns_SendCmd(u32 cmd, u16 nbits);
void sns_ReadCoeffs(void);
void sns_TriggerPressureSample(void);
void sns_TriggerTemperatureSample(void);
void sns_ConvertPressureTemperature(void);
void sns_ReadSample(void);
void sns_InitZBuf(void);
void sns_ReadSampleContinuous(void);

s32  var_Pa2Cm(s32 pa);
void var_ClimbRate();
void var_Beep(void);
void var_SetFreq(u32 nFreqHz);

//This file contains the function prototypes for the i2c function
#define PAGESIZE	32
#define TST_DEV_ID   ((u8)0xD0)

#ifdef I2C_HW

//Low Level Functions
void i2c_WaitBusIdle(void);
void i2c_Start(void);
void i2c_Write(unsigned char);
void i2c_Stop(void);
void i2c_Restart(void);
unsigned int i2c_GetS(unsigned char*, unsigned char);
void i2c_NotAck(void);
void i2c_Init(void);
unsigned int i2c_AckStatus(void);
unsigned int i2c_Get(void);
void i2c_Ack(void);
unsigned int i2c_PutS(unsigned char*);

//High Level Functions for Low Density Devices
void i2c_LDByteRead(unsigned char, unsigned char, unsigned char*, unsigned char);
unsigned int i2c_LDByteWrite(unsigned char, unsigned char, unsigned char);
unsigned int i2c_LDPageWrite(unsigned char, unsigned char, unsigned char*);
unsigned int i2c_LDSeqRead(unsigned char, unsigned char, unsigned char*, unsigned char);

//High Level Functions for High Density Devices
unsigned int HDByteReadI2C(unsigned char, unsigned char, unsigned char, unsigned char*, unsigned char);
unsigned int i2c_HDByteWrite(unsigned char, unsigned char, unsigned char, unsigned char);
unsigned int i2c_HDPageWrite(unsigned char, unsigned char, unsigned char, unsigned char*);
unsigned int i2c_HDSeqRead(unsigned char, unsigned char, unsigned char, unsigned char*, unsigned char);
#endif

#ifdef I2C_HW2

#define FCY 	250000L

#ifndef I2C_ACK
#define I2C_ACK 0
#endif
#ifndef I2C_NAK
#define I2C_NAK 1
#endif

#define I2C_WADDR(x) (x & 0xFE) //clear R/W bit of I2C addr
#define I2C_RADDR(x) (x | 0x01) //set R/W bit of I2C addr

void i2c_Init(void);

//I2C Operations
void configI2C1(u16 u16_FkHZ);
void startI2C1(void);
void rstartI2C1(void);
void stopI2C1(void);
void putI2C1(u8 u8_val);
u8 putNoAckCheckI2C1(u8 u8_val);
u8 getI2C1(u8 u8_ack2Send);

//I2C Transactions
void write1I2C1(u8 u8_addr,u8 u8_d1);
void write2I2C1(u8 u8_addr,u8 u8_d1, u8 u8_d2);
void writeNI2C1(u8 u8_addr,u8* pu8_data, u16 u16_cnt);
void read1I2C1 (u8 u8_addr,u8* pu8_d1);
void read2I2C1 (u8 u8_addr,u8* pu8_d1, u8* pu8_d2);
void readNI2C1 (u8 u8_addr,u8* pu8_data, u16 u16_cnt);
#endif

#ifdef I2C_SW2
#define I2CERR_NAK          11
int m_error;
#define SCL			PORTBbits.RB8		//I2C Clock PORTB pin 17
#define SDA			PORTBbits.RB9		//I2C Data PORTB pin 18
#define	SDA_TRIS	TRISBbits.TRISB9	//SDA Tris bit

void i2c_Init(void);
u8 i2c_RcvByte(u8 devId, u8 addr);
int i2c_XmtByte(u8 devId, u8 addr, u8 data);
int i2c_RcvBuf(u8 devId, u8 addr, int nBytes, u8* pBuf);
int i2c_XmtBuf(u8 devId, u8 addr, int nBytes, u8* pBuf );
u8 i2c_RByte(int ack);
int i2c_XByte(u8 dat);
int i2c_RBit(void);
int i2c_XBit(int bit);
int i2c_Stop(void);
int i2c_Start(void);
#endif

#ifdef I2C_SW
unsigned char control;              // Control byte variable
unsigned int address;               // Address word variable

//These are the definitions for the I2C pins that are used for Bit-Bang 
//communications.  These can be changed in the final Application.
#define SCL			PORTBbits.RB8		//I2C Clock PORTB pin 8
#define SDA			PORTBbits.RB9		//I2C Data PORTB pin 9
#define	SDA_TRIS	TRISBbits.TRISB9	//SDA Tris bit

#define     CONTROLBYTE 0xD0      // Control byte
#define     ACKBIT      0x00            // ACK bit
#define     NAKBIT      0x80            // NAK bit

void i2c_Init(void);
void bstart(void);                      // Start condition
void bstop(void);                       // Stop condition
unsigned char byte_out(unsigned char);  // Byte output
unsigned char byte_in(unsigned char);   // Byte input
void LowDensByteWrite(unsigned char);   // Low-density byte write
void HighDensByteWrite(unsigned char);  // High-density byte write
void LowDensPageWrite(unsigned char*,unsigned char);    // Low-density page write
void HighDensPageWrite(unsigned char*,unsigned char);   // High-density page write
void LowDensByteRead(unsigned char*);   // Low-density byte read
void HighDensByteRead(unsigned char*);  // High-density byte read
void LowDensSequentialRead(unsigned char*,unsigned int);    // Low-density sequential read
void HighDensSequentialRead(unsigned char*,unsigned int);   // High-density sequential read
void ACK_Poll(void);                    // Acknowledge polling

//Function Prototypes
void bit_in(unsigned char *data);		//Bit Input function
void bit_out(unsigned char data);		//Bit Out function

#endif

#ifdef DEBUG
void uart_Init(void);
void uart_Print(char *szBuf);
void dbg_PrintAltitude(s16 alt);
void dbg_PrintTemperature(s16 temp);
void dbg_PrintClimbRate(s16 cps);
#endif // DEBUG

int main(void)
	{
	u8 xmt,rcv;
    mcu_Init();
    //sns_Init();
#ifdef DEBUG
#endif
  
       
    #ifdef I2C_SW
    address = 0;
    //LowDensByteWrite(12);
    #endif

    #ifdef I2C_SW2
    #endif

    xmt = 0;
    while(1)
		{
	    mcu_Delay();
       #ifdef I2C_HW
    	i2c_LDByteRead(0xD0,0,&rcv,1);
       #endif
       #ifdef I2C_HW2
       write1I2C1(0xD0, 0);
       read1I2C1 (0xD0, &rcv);
       #endif
       #ifdef I2C_SW
    	LowDensByteRead(&rcv);
       #endif
       #ifdef I2C_SW2
       //i2c_XmtByte(0xD0,0x06, xmt++);
       mcu_Delay();
       rcv = i2c_RcvByte(0xD0,0x0);
       #endif
        sprintf(gszBuf,"%d ",(int)rcv);
        uart_Print(gszBuf);
		}

  	//sns_TriggerTemperatureSample();
   	//sns_ReadSampleContinuous();

    return 0;
	}


void mcu_Init(void)
	{
    RPINR18bits.U1RXR = 5; // pin14
    RPOR2bits.RP4R = FUNC_UART1_TX; // pin 11
    RPOR7bits.RP15R = FUNC_OC1; // pin 26
    
//	OSCCONbits.IOLOCK = 1;
    CLKDIVbits.RCDIV = 3;  // divide by 8 => FOsc = 500KHz

    ODCBbits.ODB4 = 0;  // digital u1tx pin11
    ODCBbits.ODB5 = 0;  // digital u1rx pin14
    ODCBbits.ODB15 = 0; // digital oc1 pin26
    
    TRISBbits.TRISB4 = 0; // output u1tx
    TRISBbits.TRISB5 = 1; // input u1rx
    TRISBbits.TRISB15 = 0; // output oc1
 
    T2CONbits.T32 = 0; // 16bit timer
    T2CONbits.TCS = 0; //  select internal clock 
    T2CONbits.TGATE = 0; // gate disabled
    T2CONbits.TSIDL = 0; // continue in idle mode
    T2CONbits.TCKPS1 = 0; // Prescaler 1:1
    T2CONbits.TCKPS0 = 0; // Prescaler 1:1
    OC1CONbits.OCM = 5;  // continuous pulse mode
    OC1CONbits.OCTSEL = 0; // use TMR2

    i2c_Init();
#ifdef DEBUG
    uart_Init();
#endif
	}


void mcu_Delay(void)
	{
    unsigned long m,n;
    m = 20;
    while(m--) {
       	n = 2000;
       	while(n--);
       	}
    }



void var_SetFreq(u32 nFreqHz)
	{
    u16 tmrTicks;
    T2CONbits.TON = 0; // stop tmr2
    if (nFreqHz) {
    	tmrTicks = (u16)(FOSC_HZ/nFreqHz);
    	OC1RS = tmrTicks;
    	OC1R = (tmrTicks >> 1); // square wave
    	PR2 = (tmrTicks+1);
		}
    else { // dc 0V
        OC1RS = 100;
        OC1R = 50;
        PR2 = 40;
        }
    T2CONbits.TON = 1; // start tmr2
    }

    
void sns_ReadSample(void)
   	{
   	sns_TriggerTemperatureSample();
   	while (gpioMDOUT);
   	gwD2 = sns_ReadWord(); // read temperature
   	sns_TriggerPressureSample();
   	while (gpioMDOUT);
   	gwD1 = sns_ReadWord(); // read pressure
   	sns_ConvertPressureTemperature();
   	gnAltCm = var_Pa2Cm(gP*10);
   	gnTempCx10 = gT;
#ifdef DEBUG
   	dbg_PrintAltitude((gnAltCm+50)/100); 
   	dbg_PrintTemperature(gnTempCx10);
#endif
   	}

void sns_InitZBuf(void)
   	{
   	u16 cnt = SNS_NUM_Z_SAMPLES;
   	sns_ReadSample();
   	while (cnt--)  {
      	gZBuf[cnt] = gnAltCm;
      	}
   	}

void sns_Init(void)
   	{
  	gnAcqState = 0;
   	gnSmpCnt = 0;
   	gnCps = 0;
   	sns_Reset();
   	sns_ReadCoeffs();
   	sns_InitZBuf();
   	}


void sns_Reset(void)
   	{
   	}


u16 sns_ReadWord(void)
   	{
   	u16 w;
   	return w;
   	}

void sns_SendCmd(u32 cmd, u16 nbits)
   	{
   	}

u16 sns_ReadCoeff(u8 addr)
   	{
   	}


void sns_ReadCoeffs(void)
   	{
#ifdef DEBUG
   	sprintf(gszBuf,"%u %u %u %u %u %u",gwC1,gwC2,gwC3,gwC4,gwC5,gwC6);
    uart_Print(gszBuf);
#endif
   	}


void sns_TriggerPressureSample(void)
   	{
   	}

void sns_TriggerTemperatureSample(void)
   	{
   	}


void sns_ConvertPressureTemperature(void)
    {

    }


void sns_ReadSampleContinuous(void)
   	{  
	while (1) {
		sprintf(gszBuf,"%d ",gnSmpCnt++);
        uart_Print(gszBuf);
        mcu_Delay();
		}
/*
   		if (gpioMDOUT == 0) { // conversion complete
   			if (gnAcqState)  {
        		//gpioLED = !gpioLED;
      			gwD1 = sns_ReadWord(); // read pressure
      			sns_TriggerTemperatureSample();
      			sns_ConvertPressureTemperature();
      			// ms5534 has 0.1mbar (10pa) resolution, LUT has 1 pa input resolution. 
      			gZBuf[gnSmpCnt] = gnAltCm = var_Pa2Cm(gP*10);
      			gnTempCx10 = gT;
      			var_ClimbRate();
      			var_Beep();
      			gnSmpCnt++;
      			if (gnSmpCnt == SNS_NUM_Z_SAMPLES) {
         			gnSmpCnt = 0;
         			}
      			gnAcqState = 0;
      			}
   			else {
      			gwD2 = sns_ReadWord(); // read temperature
      			sns_TriggerPressureSample();
#ifdef DEBUG
      			//dbg_PrintAltitude((gnAltCm+50)/100); 
      			//dbg_PrintClimbRate(gnCps);
      			//dbg_PrintTemperature(gnTempCx10); 
#endif
      			gnAcqState = 1;
      			}
			}
   		}
*/
	}

void var_ClimbRate(void)
   	{
   	u16 n;
   	s32 sum_zt,sum_z,sum_t,sum_t2,trel,num,den,tref,z;
   	tref = gnSmpCnt; // time origin is the current sample in window
   	sum_zt = sum_z = sum_t = sum_t2 = 0;
   	n = SNS_NUM_Z_SAMPLES;
   	while (n--)  {
      	z = gZBuf[n];
      	trel = n-tref;
      	if (n > tref) trel -= SNS_NUM_Z_SAMPLES;
      	sum_z += z;
      	sum_t += trel;
      	sum_t2 += (trel*trel);
      	sum_zt += (trel*z);
      	}
   	num = (SNS_NUM_Z_SAMPLES*sum_zt) - (sum_z*sum_t);
   	den = (SNS_NUM_Z_SAMPLES*sum_t2) - (sum_t*sum_t);
   	gnCps = den ? (s16)((num*SNS_ACQ_PER_SEC)/den) : 0;
   	return;
   	}

s32 var_Pa2Cm(s32 pa)
   	{
   	s32 inx,pa1,pa2,z1,z2,z;
   	if (pa > PA_INIT)  {
      	z = gnPZTbl[0];
      	}
   	else {
      	inx = (PA_INIT - pa)>>10;      
      	if (inx >= PZLUT_ENTRIES) {
         	z = gnPZTbl[PZLUT_ENTRIES-1];
         	}
      	else {
         	pa2 = PA_INIT - (inx<<10);
         	pa1 = (inx > 0 ? pa2+PA_DELTA : pa2);
         	z1 = (inx > 0 ? gnPZTbl[inx-1] : gnPZTbl[0]);
         	z2 = gnPZTbl[inx];
         	z = ((pa1-pa)*z2 + (pa-pa2)*z1)>>10;
         	}
      	}
   	return z;
   	}





void var_Beep(void)
	{
    u32 nFreqHz;
   	if (gnBeepPeriodTicks <= 0 || (ABS(gnCps-gnVarioCps) > BEEP_DISCR_THRESHOLD) || ((gnCps > 0) && (gnVarioCps <= 0)) || ((gnCps <= 0) && (gnVarioCps > 0))) {
   		gnVarioCps = gnCps;
      	if (gnVarioCps < VARIO_SINKTONE_CPS){
            var_SetFreq(1000);
         	gnBeepPeriodTicks = 0;
         	gnBeepTransition  = 0;
         	}
      	else{    
           	gnBeepCps = gnVarioCps;
         	if (gnBeepCps > 0) {
            	gnBeepPeriodTicks =  (gnBeepCps > 768) ? 3 :  15 - (gnBeepCps>>6);
            	nFreqHz =  VARIO_MINFREQ_HZ + ((u32)gnBeepCps*(VARIO_MAXFREQ_HZ - VARIO_MINFREQ_HZ))/VARIO_MAXCPS;
                if (nFreqHz > VARIO_MAXFREQ_HZ) nFreqHz = VARIO_MAXFREQ_HZ;
            	gnBeepTransition = (gnBeepPeriodTicks<<1)/3;   
            	var_SetFreq(nFreqHz);
            	}
         	else{
                var_SetFreq(0);
            	}
         	}
   		}
   	else{
      	gnBeepPeriodTicks--;
      	if (gnBeepPeriodTicks <= gnBeepTransition){
         	var_SetFreq(0);
         	}
      	}       	
   }


#ifdef I2C_SW2
void i2c_Init(void)
	{
    //ODCBbits.ODB8 = 1;           // open drain scl pin17
    //ODCBbits.ODB9 = 1;           // open drain sda pin18
	TRISBbits.TRISB8 = 0;		 // SCL output.
	TRISBbits.TRISB9 = 0;        // SDA output
    Nop();
    SDA = 1;
    SCL = 1;
    m_error = 0;
	}


int i2c_Start(void)
    {
    SCL = 1; //  SCL is high
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    SDA_TRIS = 0;
    Nop();
    SDA = 0;  // Pull SDA down while SCL is high ... start condition
    Nop();
    SCL = 0; //  Pull SCL low for next bit
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    return 0;
    }

int i2c_Stop(void)
    {
    SDA_TRIS = 0;
    Nop();
    SDA = 0;  // Pull SDA low
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    Nop();
    SCL = 1;   // Let SCL go up
    Nop();
    SDA = 1;  // then SDA
    //SDA_TRIS = 1;
    Nop();
    return 0;
    }

int i2c_XBit(int bit)
   {
   SDA_TRIS = 0;
   Nop();
   SDA = (bit ? 1 : 0);
   Nop();
   SCL = 1;
   Nop();
   SCL = 0;
   Nop();
   return 0;
   }

int i2c_RBit(void)
   {
   int bit;
   SDA_TRIS = 1;
   Nop();
   SCL = 1;
   Nop();
   bit = (SDA ? 1 : 0);  
   SCL = 0;
   Nop();
   return bit;
   }

int i2c_XByte(u8 dat)
    {
    int bitCount;

    bitCount = 8;
    while(bitCount)
        {
        if (dat & 0x80)
            {
            i2c_XBit(1);
            }
        else
            {
            i2c_XBit(0);
            }
        dat <<= 1;
        bitCount--;
        }

    if (i2c_RBit())
        {
        m_error = I2CERR_NAK;
        return 1;
        }
    return 0;
    }

u8 i2c_RByte(int ack)
    {
    int bitCount;
    u8 inByte;

    bitCount = 8;
    inByte = 0;

    while(bitCount)
        {
        inByte <<= 1;
        inByte |= i2c_RBit();
        bitCount--;
        }

    // acknowledge
    i2c_XBit(ack);
    return inByte;
    }


int i2c_XmtBuf(u8 devId, u8 addr, int nBytes, u8* pBuf )
    {
    int cnt;

    m_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        return 0xF1;
        }
    for (cnt = 0; cnt < nBytes; cnt++)
        if (i2c_XByte(pBuf[cnt]))
            {
            return 0xF2;
            }
    i2c_Stop();
    return 0;
    }




int i2c_RcvBuf(u8 devId, u8 addr, int nBytes, u8* pBuf)
    {
    int cnt;

    m_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        if (m_error == I2CERR_NAK) i2c_Stop();
        return 0xF1;
        }
    SDA_TRIS = 0;
    Nop();
    SDA = 1; // Set SDA high while SCL is low
    Nop(); Nop(); Nop(); Nop();
    // Restart
    i2c_Start();
    if (i2c_XByte(devId|0x1))
        {
        return 0xF2;
        }

    for (cnt = 0; cnt < nBytes-1; cnt++)
        {
        pBuf[cnt] = i2c_RByte(0);
        }
    // Master NAK
    pBuf[nBytes-1] = i2c_RByte(1);
    i2c_Stop();
    return (m_error ? 1 : 0);
    }



int i2c_XmtByte(u8 devId, u8 addr, u8 data)
    {
    m_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        return 0xF1;
        }
    if (i2c_XByte(data))
        {
        return 0xF2;
        }
    i2c_Stop();
    return 0;
    }




u8 i2c_RcvByte(u8 devId, u8 addr)
    {
    u8 data;
    
    m_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))
        {
        return 0xF0;
        }
    if (i2c_XByte(addr))
        {
        if (m_error == I2CERR_NAK) i2c_Stop();
        return 0xF1;
        }
    SDA_TRIS = 0;
    Nop();
    SDA = 1; // Set SDA high while SCL is low
    // Restart
    i2c_Start();
    if (i2c_XByte(devId|0x1))
        {
        return 0xF2;
        }
    data = i2c_RByte(1);
    i2c_Stop();
    return data;
    }

#endif

#ifdef I2C_SW

void i2c_Init(void)
	{
   // ODCBbits.ODB8 = 1;           // open drain scl pin17
	TRISBbits.TRISB8 = 0;		 // SCL Line output.

  //  ODCBbits.ODB9 = 1;           // open drain sda pin18
	TRISBbits.TRISB9 = 1;        // SDA line input

    control = 0xD0;
    address = 0;
	}

/********************************************************************
 * Function:        void bstart(void)
 *
 * Description:     This function generates an I2C Start condition.
 *******************************************************************/
void bstart(void)
{
    SDA_TRIS = 1;                   // Ensure SDA is high
    SCL = 1;                        // Ensure SCL is high
	Nop();
    SDA_TRIS = 0;                   // Configure SDA as an output
    SDA = 0;                        // Pull SDA low
	Nop();
    SCL = 0;                        // Pull SCL low
    
} // end bstart(void)

/********************************************************************
 * Function:        void bstop(void)
 *
 * Description:     This function generates an I2C Stop condition.
 *******************************************************************/
void bstop(void)
{
    SCL = 0;                        // Ensure SCL is low
    SDA_TRIS = 0;                   // Configure SDA as an output
    SDA = 0;                        // Ensure SDA low
	Nop();
    SCL = 1;                        // Pull SCL high
	Nop();
    SDA_TRIS = 1;                   // Allow SDA to be pulled high
} // end bstop(void)

/********************************************************************
 * Function:        void bit_out(unsigned char data)
 *
 * Description:     This function outputs a bit to the I2C bus.
 *******************************************************************/
void bit_out(unsigned char data)
{
    SCL = 0;                        // Ensure SCL is low
    if (data & 0x80)                // Check if next bit is high
    {
        SDA_TRIS = 1;               // Release SDA to be pulled high
		Nop();
		Nop();
    }
    else
    {
        SDA_TRIS = 0;               // Configure SDA as an output
        SDA = 0;                    // Pull SDA low
		Nop();
		Nop();
    }
    SCL = 1;                        // Pull SCL high to clock bit
	Nop();
	Nop();
	Nop();
    SCL = 0;                        // Pull SCL low for next bit
} // end bit_out(unsigned char data)

/********************************************************************
 * Function:        void bit_in(unsigned char *data)
 *
 * Description:     This function inputs a bit from the I2C bus.
 *******************************************************************/
void bit_in(unsigned char *data)
{
    SCL = 0;                        // Ensure SCL is low	
	Nop();
    SDA_TRIS = 1;                   // Configure SDA as an input
	Nop();
    SCL = 1;                        // Bring SCL high to begin transfer
    *data &= 0xFE;                  // Assume next bit is low
    if (SDA)                        // Check if SDA is high
    {
        *data |= 0x01;              // If high, set next bit
    }
	Nop();
    SCL = 0;                        // Bring SCL low again
} // end bit_in(unsigned char *data)

/********************************************************************
 * Function:        unsigned char byte_out(unsigned char data)
 *
 * Description:     This function outputs a byte to the I2C bus.
 *                  It also receives the ACK bit and returns 0 if
 *                  successfully received, or 1 if not.
 *******************************************************************/
unsigned char byte_out(unsigned char data)
{
    unsigned char i;                // Loop counter
    unsigned char ack;              // ACK bit

    ack = 0;
    for (i = 0; i < 8; i++)         // Loop through each bit
    {
        bit_out(data);              // Output bit
        data = data << 1;           // Shift left for next bit
    }
    bit_in(&ack);                   // Input ACK bit

    return ack;
} // end byte_out(unsigned char data)

/********************************************************************
 * Function:        unsigned char byte_in(unsigned char ack)
 *
 * Description:     This function inputs a byte from the I2C bus.
 *                  Depending on the value of ack, it will also
 *                  transmit either an ACK or a NAK bit.
 *******************************************************************/
unsigned char byte_in(unsigned char ack)
{
    unsigned char i;                // Loop counter
    unsigned char retval;           // Return value

    retval = 0;
    for (i = 0; i < 8; i++)         // Loop through each bit
    {
        retval = retval << 1;       // Shift left for next bit
        bit_in(&retval);            // Input bit
    }
    bit_out(ack);                   // Output ACK/NAK bit

    return retval;
} // end byte_in(void)

/********************************************************************
 * Function:        void LowDensByteWrite(unsigned char data)
 *
 * Description:     This function writes a single byte to a
 *                  low-density (<= 16 Kb) serial EEPROM device.
 *
 * Dependencies:    'control' contains the control byte
 *                  'address' contains the address word
 *******************************************************************/
void LowDensByteWrite(unsigned char data)
{
    unsigned char temp_control;     // Temp. variable for control byte

    // Merge block bits with control byte
//    temp_control = (control & 0xF1) | ((address >> 7) & 0x0E);
    temp_control = control;

    bstart();                       // Generate Start condition
    byte_out(temp_control);         // Output control byte
    byte_out((unsigned char)address);// Output address LSB
    byte_out(data);                 // Output data byte
    bstop();                        // Generate Stop condition
    ACK_Poll();                     // Begin ACK polling
} // end LowDensByteWrite(unsigned char data)



/********************************************************************
 * Function:        void LowDensPageWrite(unsigned char *data,
 *                                        unsigned char numbytes)
 *
 * Description:     This function writes multiple bytes to a
 *                  low-density (<= 16 Kb) serial EEPROM device.
 *
 * Dependencies:    'control' contains the control byte
 *                  'address' contains the address word
 *******************************************************************/
void LowDensPageWrite(unsigned char *data, unsigned char numbytes)
{
    unsigned char i;                // Loop counter
    unsigned char temp_control;     // Temp. variable for control byte

    // Merge block bits with control byte
//    temp_control = (control & 0xF1) | ((address >> 7) & 0x0E);
    temp_control = control;

    bstart();                       // Generate Start condition
    byte_out(temp_control);         // Output control byte
    byte_out((unsigned char)address);// Output address LSB
    for (i = 0; i < numbytes; i++)  // Loop through data bytes
    {
        byte_out(data[i]);          // Output next data byte
    }
    bstop();                        // Generate Stop condition
    ACK_Poll();                     // Begin ACK polling
} // end LowDensPageWrite(unsigned char *data, unsigned char numbytes)



/********************************************************************
 * Function:        void LowDensByteRead(unsigned char *data)
 *
 * Description:     This function reads a single byte from a
 *                  low-density (<= 16 Kb) serial EEPROM device.
 *
 * Dependencies:    'control' contains the control byte
 *                  'address' contains the address word
 *******************************************************************/
void LowDensByteRead(unsigned char *data)
{
    unsigned char temp_control;     // Temp. variable for control byte

    // Merge block bits with control byte
//    temp_control = (control & 0xF1) | ((address >> 7) & 0x0E);
    temp_control = control;

    bstart();                       // Generate Start condition
    byte_out(temp_control);         // Output control byte
    byte_out((unsigned char)address);// Output address LSB
    bstart();                       // Generate Start condition
    byte_out(temp_control | 0x01);  // Output control byte
    *data = byte_in(NAKBIT);        // Input data byte
    bstop();                        // Generate Stop condition
} // end LowDensByteRead(unsigned char data)



/********************************************************************
 * Function:        void LowDensSequentialRead(unsigned char *data,
 *                                             unsigned char numbytes)
 *
 * Description:     This function reads multiple bytes from a
 *                  low-density (<= 16 Kb) serial EEPROM device.
 *
 * Dependencies:    'control' contains the control byte
 *                  'address' contains the address word
 *******************************************************************/
void LowDensSequentialRead(unsigned char *data, unsigned int numbytes)
{
    unsigned int i;                 // Loop counter
    unsigned char temp_control;     // Temp. variable for control byte

    // Merge block bits with control byte
    temp_control = (control & 0xF1) | ((address >> 7) & 0x0E);

    bstart();                       // Generate Start condition
    byte_out(temp_control);         // Output control byte
    byte_out((unsigned char)address);// Output address LSB
    bstart();                       // Generate Start condition
    byte_out(temp_control | 0x01);  // Output control byte
    for (i = 0; i < numbytes; i++)  // Loop through data bytes
    {
        if (i < (numbytes - 1))     // Check if more data will be read
        {
            data[i] = byte_in(ACKBIT); // If not last, input byte & send ACK
        }
        else
        {
            data[i] = byte_in(NAKBIT); // If last byte, input byte & send NAK
        }
    }
    bstop();                        // Generate Stop condition
} // end LowDensSequentialRead(unsigned char *data, unsigned char numbytes)



/********************************************************************
 * Function:        void ACK_Poll(void)
 *
 * Description:     This function implements Acknowledge polling.
 *
 * Dependencies:    'control' contains the control byte
 *******************************************************************/
void ACK_Poll(void)
{
    unsigned char result;           // Polling result

    result = 1;                     // Initialize result
    do
    {
        bstart();                   // Generate Start condition
        result = byte_out(control); // Output control byte
    } while (result == 1);
    bstop();                        // Generate Stop condition
} // end ACK_Poll(void)

#endif

#ifdef I2C_HW

void i2c_Init(void)
	{	
    //ODCBbits.ODB8 = 1;  // open drain  pin17
    //ODCBbits.ODB9 = 1;  // open drain  pin18
    //TRISBbits.TRISB8 = 1;
    //TRISBbits.TRISB9 = 1;

	//About 75Khz at Fosc 500khz
	I2C1BRG = 0x0002; 

	//Master Mode, No Slew Rate Control, and leave the peripheral switched off.
	I2C1CON = 0x1200;
	
	I2C1RCV = 0x0000;
	I2C1TRN = 0x0000;

	//Now we can enable the peripheral	
	I2C1CON = 0x9200;
	}


void i2c_Start(void)
	{
	I2C1CONbits.SEN = 1;		//Generate Start COndition
	while (I2C1CONbits.SEN);	//Wait for Start COndition
	}


void i2c_Restart(void)
	{
	I2C1CONbits.RSEN = 1;		//Generate Restart		
	while (I2C1CONbits.RSEN);	//Wait for restart	
	}


void i2c_Stop(void)
	{
	I2C1CONbits.PEN = 1;		//Generate Stop Condition
	while (I2C1CONbits.PEN);	//Wait for Stop
	}


void i2c_Write(unsigned char byte)
	{
	I2C1TRN = byte;					//Load byte to I2C1 Transmit buffer
	while (I2C1STATbits.TRSTAT);		//wait for data transmission
	}


void i2c_WaitBusIdle(void)
	{
	while (I2C1STATbits.TRSTAT);		//Wait for bus Idle
	}


unsigned int i2c_LDByteWrite(unsigned char deviceID, unsigned char LowAdd, unsigned char data)
	{
	unsigned int ErrorCode;

	i2c_WaitBusIdle();						//Ensure Module is Idle
	i2c_Start();						//Generate Start COndition
	i2c_Write(deviceID);			//Write Control byte
//	i2c_WaitBusIdle();

	ErrorCode = i2c_AckStatus();		//Return ACK Status
	
	i2c_Write(LowAdd);				//Write Low Address
//	i2c_WaitBusIdle();

	ErrorCode = i2c_AckStatus();		//Return ACK Status

	i2c_Write(data);					//Write Data
//	i2c_WaitBusIdle();
	i2c_Stop();						//Initiate Stop Condition
	return(ErrorCode);
	}


void i2c_LDByteRead(unsigned char deviceID, unsigned char Address, unsigned char *Data, unsigned char Length)
	{
	//i2c_WaitBusIdle();					//wait for bus Idle
	i2c_Start();					//Generate Start Condition
	i2c_Write(deviceID);		//Write device ID
	//i2c_WaitBusIdle();					//wait for bus Idle
	i2c_Write(Address);			//Write start address
	//i2c_WaitBusIdle();					//wait for bus Idle

	i2c_Restart();				//Generate restart condition
	i2c_Write(deviceID | 0x01);	//Write control byte for read
	//i2c_WaitBusIdle();					//wait for bus Idle

	i2c_GetS(Data, Length);		//read Length number of bytes
	i2c_NotAck();				//Send Not Ack
	i2c_Stop();					//Generate Stop
	}


unsigned int i2c_HDByteWrite(unsigned char deviceID, unsigned char HighAdd, unsigned char LowAdd, unsigned char data)
	{
	unsigned int ErrorCode;

	i2c_WaitBusIdle();						//Ensure Module is Idle
	i2c_Start();						//Generate Start COndition
	i2c_Write(deviceID);			//Write device ID
	i2c_WaitBusIdle();

	ErrorCode = i2c_AckStatus();		//Return ACK Status
	
	i2c_Write(HighAdd);
	i2c_WaitBusIdle();						//Write High Address
	i2c_Write(LowAdd);				//Write Low Address
	i2c_WaitBusIdle();

	ErrorCode = i2c_AckStatus();		//Return ACK Status

	i2c_Write(data);					//Write Data
	i2c_WaitBusIdle();
	i2c_Stop();						//Initiate Stop Condition
	return(ErrorCode);
	}


unsigned int HDByteReadI2C(unsigned char deviceID, unsigned char HighAdd, unsigned char LowAdd, unsigned char *Data, unsigned char Length)
	{
	i2c_WaitBusIdle();						//Wait for bus Idle
	i2c_Start();						//Generate Start condition
	i2c_Write(deviceID);			//send device ID for write
	i2c_WaitBusIdle();						//Wait for bus Idle

	i2c_Write(HighAdd);				//Send High Address
	i2c_WaitBusIdle();						//Wait for bus Idle
	i2c_Write(LowAdd);				//Send Low Address
	i2c_WaitBusIdle();						//Wait for bus Idle

	i2c_Restart();					//Generate Restart
	i2c_Write(deviceID | 0x01);	//send device id for Read
	i2c_WaitBusIdle();						//Wait for bus Idle

	i2c_GetS(Data, Length);			//Read Length number of bytes to Data
	i2c_NotAck();					//send Not Ack
	i2c_Stop();						//Send Stop Condition
	return(0);
	}





unsigned int i2c_LDSeqRead(unsigned char deviceID, unsigned char address, unsigned char *rdptr, unsigned char length)
	{
	i2c_WaitBusIdle();						//Ensure Module is Idle
	i2c_Start();						//Initiate start condition
	i2c_Write(deviceID);			//write device id byte
	i2c_WaitBusIdle();						//Ensure module is Idle
	i2c_Write(address);				//Write word address
	i2c_WaitBusIdle();						//Ensure module is idle
	i2c_Restart();					//Generate I2C Restart Condition
	i2c_Write(deviceID | 0x01);	//Write 1 byte - R/W bit should be 1 for read
	i2c_WaitBusIdle();						//Ensure bus is idle
	i2c_GetS(rdptr, length);			//Read in multiple bytes
	i2c_NotAck();					//Send Not Ack
	i2c_Stop();						//Send stop condition
	return(0);
	}


unsigned int i2c_HDSeqRead(unsigned char deviceID, unsigned char HighAdd, unsigned char LowAdd, unsigned char *rdptr, unsigned char length)
	{
	i2c_WaitBusIdle();						//Ensure Module is Idle
	i2c_Start();						//Initiate start condition
	i2c_Write(deviceID);			//write device id
	i2c_WaitBusIdle();						//Ensure module is Idle
	i2c_Write(HighAdd);				//Write High word address
	i2c_WaitBusIdle();						//Ensure module is idle
	i2c_Write(LowAdd);				//Write Low word address
	i2c_WaitBusIdle();						//Ensure module is idle
	i2c_Restart();					//Generate I2C Restart Condition
	i2c_Write(deviceID | 0x01);	//Write 1 byte - R/W bit should be 1 for read
	i2c_WaitBusIdle();						//Ensure bus is idle
	i2c_GetS(rdptr, length);			//Read in multiple bytes
	i2c_NotAck();					//Send Not Ack
	i2c_Stop();						//Send stop condition
	return(0);
	}


unsigned int i2c_AckStatus(void)
	{
	return (!I2C1STATbits.ACKSTAT);		//Return Ack Status
	}


void i2c_NotAck(void)
	{
	I2C1CONbits.ACKDT = 1;			//Set for NotACk
	I2C1CONbits.ACKEN = 1;
	while(I2C1CONbits.ACKEN);		//wait for ACK to complete
	I2C1CONbits.ACKDT = 0;			//Set for NotACk
	}


void i2c_Ack(void)
	{
	I2C1CONbits.ACKDT = 0;			//Set for ACk
	I2C1CONbits.ACKEN = 1;
	while(I2C1CONbits.ACKEN);		//wait for ACK to complete
	}


unsigned int i2c_GetS(unsigned char *rdptr, unsigned char Length)
	{
	while (Length --)
		{
		*rdptr++ = i2c_Get();		//get a single byte
		
		if(I2C1STATbits.BCL)		//Test for Bus collision
			{
			return(-1);
			}

		if(Length)
			{
			i2c_Ack();				//Acknowledge until all read
			}
		}
	return(0);
	}


unsigned int i2c_Get(void)
	{
	I2C1CONbits.RCEN = 1;			//Enable Master receive
	Nop();
	while(!I2C1STATbits.RBF);		//Wait for receive bufer to be full
	return(I2C1RCV);				//Return data in buffer
	}


unsigned int i2c_PutS(unsigned char *wrptr)
	{
	unsigned char x;

	for(x = 0; x < PAGESIZE; x++)		//Transmit Data Until Pagesize
		{	
		i2c_Write(*wrptr);			//Write 1 byte
		i2c_WaitBusIdle();					//Wait for Idle bus
		if(I2C1STATbits.ACKSTAT)
			{
			return(-2);				//Bus responded with Not ACK
			}
		wrptr++;
		}
	return(0);
	}

#endif


#ifdef I2C_HW2

void i2c_Init(void)
	{
    configI2C1(100);
	}

/**
Configure and enable the I2C1 module for operation at \em u16_FkHZ kHZ clock speed.
\param u16_FkHZ specifies clock speed in kHZ
*/
void configI2C1(u16 u16_FkHZ) {
  u32 u32_temp;

  u32_temp = (FCY/1000L)/((u32) u16_FkHZ);
  u32_temp = u32_temp - FCY/10000000L - 1;
  I2C1BRG = u32_temp;
  I2C1CONbits.I2CEN = 1;
}

/**
Operation: Perform an I2C start operation.
*/
void startI2C1(void) {
  I2C1CONbits.SEN = 1;   // initiate start
  // wait until start finished
  while (I2C1CONbits.SEN);
}

/**
Operation: Perform an I2C repeated start operation.
*/
void rstartI2C1(void) {// repeated start
  I2C1CONbits.RSEN = 1;   // initiate start
  // wait until start finished
  while (I2C1CONbits.RSEN);
}

/**
Operation: Perform an I2C stop operation.
*/
void stopI2C1(void) {
  I2C1CONbits.PEN=1;     // initiate stop, PEN=1
  //wait until stop finished
  while (I2C1CONbits.PEN);
}

/**
Operation: Send one byte (\em u8_val), if NAK is returned use reportError() function to save error and do software reset.
\param u8_val byte to send
*/
void putI2C1(u8 u8_val) {
  I2C1TRN = u8_val;    // write byte
  while (I2C1STATbits.TRSTAT);   // wait for 8bits+ ack bit to finish
  if (I2C1STATbits.ACKSTAT != I2C_ACK) {
    //NAK returned
  }
}

/**
Operation: Send one byte (\em u8_val), return the acknowledgement bit that comes back from the slave. This
function does not error out if a NAK is returned.
\param u8_val byte to send
\return Ack bit value returned from slave.
*/
u8 putNoAckCheckI2C1(u8 u8_val) {
  I2C1TRN = u8_val;    // write byte
  while (I2C1STATbits.TRSTAT);   // wait for 8bits+ ack bit to finish
  return(I2C1STATbits.ACKSTAT);
}

/**
Operation: Wait for a byte byte on the I2C bus, send \em u8_ack2Send as the acknowledgement bit to send back to the slave.
\param u8_ack2Send ack bit to send back to slave after byte is read
\return byte read from slave
*/
u8 getI2C1(u8 u8_ack2Send) {
  u8 u8_inByte;

  while (I2C1CON & 0x1F);           //wait for idle condition
  I2C1CONbits.RCEN = 1;             //enable receive
  while (!I2C1STATbits.RBF);        //wait for receive byte
  u8_inByte = I2C1RCV;              //read byte;
  //wait for idle condition before attempting ACK
  while (I2C1CON & 0x1F);           //lower 5 bits must be 0
  I2C1CONbits.ACKDT = u8_ack2Send;  //ACK bit to send back on receive
  I2C1CONbits.ACKEN = 1;            //enable ACKbit transmittion
  while (I2C1CONbits.ACKEN);        //wait for completion
  return(u8_inByte);                  //return the value
}
/**
Transaction: Write 1 byte (\em u8_d1) to I2C slave at address \em u8_addr.
\param u8_addr  Slave I2C address
\param u8_d1 Byte to send
*/
void write1I2C1(u8 u8_addr,u8 u8_d1) {
  startI2C1();
  putI2C1(I2C_WADDR(u8_addr));
  putI2C1(u8_d1);
  stopI2C1();
}
/**
Transaction: Write 2 bytes (\em u8_d1, \em u8_d2) to I2C slave at address \em u8_addr.
\param u8_addr  Slave I2C address
\param u8_d1 First byte to send
\param u8_d2 Second byte to send
*/
void write2I2C1(u8 u8_addr,u8 u8_d1, u8 u8_d2) {
  startI2C1();
  putI2C1(I2C_WADDR(u8_addr));
  putI2C1(u8_d1);
  putI2C1(u8_d2);
  stopI2C1();
}

/**
Transaction: Write \em u16_cnt bytes stored in buffer \em *pu8_data to I2C slave at address \em u8_addr.
\param u8_addr  Slave I2C address
\param pu8_data Pointer to buffer containing bytes to send
\param u16_cnt Number of bytes to send
*/
void writeNI2C1(u8 u8_addr,u8* pu8_data, u16 u16_cnt) {
  u16 u16_i;
  startI2C1();
  putI2C1(I2C_WADDR(u8_addr));
  for (u16_i=0; u16_i < u16_cnt; u16_i++) {
    putI2C1(*pu8_data);
    pu8_data++;
  }
  stopI2C1();
}
/**
Transaction: Read one byte from I2C slave at address \em u8_addr, save to \em *pu8_d1.
As per the I2C standard, a NAK is returned for the last byte read from the slave, ACKs are returned for the other bytes.
\param u8_addr  Slave I2C address
\param pu8_d1 Pointer to location to store byte read from slave
*/
void read1I2C1 (u8 u8_addr,u8* pu8_d1) {
  startI2C1();
  putI2C1(I2C_RADDR(u8_addr));
  *pu8_d1 = getI2C1(I2C_NAK); //last ack bit from master to slave during read must be a NAK
  stopI2C1();
}
/**
Transaction: Read two bytes from I2C slave at address \em u8_addr, save to \em *pu8_d1, \em *pu8_d2.
As per the I2C standard, a NAK is returned for the last byte read from the slave, ACKs are returned for the other bytes.
\param u8_addr  Slave I2C address
\param pu8_d1 Pointer to location to store first byte read from slave
\param pu8_d2 Pointer to location to store second byte read from slave
*/
void read2I2C1 (u8 u8_addr,u8* pu8_d1, u8* pu8_d2) {
  startI2C1();
  putI2C1(I2C_RADDR(u8_addr));
  *pu8_d1 = getI2C1(I2C_ACK);
  *pu8_d2 = getI2C1(I2C_NAK);
  stopI2C1();
}
/**
Transaction: Read \em u16_cnt bytes from I2C slave at address \em u8_addr, save to buffer \em *pu8_data.
As per the I2C standard, a NAK is returned for the last byte read from the slave, ACKs are returned for the other bytes.
\param u8_addr  Slave I2C address
\param pu8_data Pointer to buffer for storing bytes read from slave
\param u16_cnt Number of bytes read from slave.
*/
void readNI2C1 (u8 u8_addr,u8* pu8_data, u16 u16_cnt) {
  u16 u16_i;
  startI2C1();
  putI2C1(I2C_RADDR(u8_addr));
  for (u16_i=0; u16_i < u16_cnt; u16_i++) {
    if (u16_i != u16_cnt-1) *pu8_data = getI2C1(I2C_ACK);
    else *pu8_data = getI2C1(I2C_NAK);
    pu8_data++;
  }
  stopI2C1();
}

#endif

#ifdef DEBUG

void uart_Init(void)
	{
    char tmp;
    U1BRG  = 3;     // 31250baud @ Fosc = 500Khz 
	U1MODEbits.BRGH = 1;
    U1MODEbits.UARTEN = 1;
    U1MODEbits.RTSMD = 1; // no flow control

    while(U1STAbits.URXDA == 1) {
        tmp = U1RXREG;
        }

    // clear IF flags 
    IFS0bits.U1RXIF = 0;
    IFS0bits.U1TXIF = 0;

    // set priority 
    IPC2bits.U1RXIP = 1;
    IPC3bits.U1TXIP = 1;

    // disable interrupt 
    IEC0bits.U1RXIE = 0;
    IEC0bits.U1TXIE = 0;
    // enable transmit
    U1STAbits.UTXEN = 1;
	}

void uart_Print(char *szBuf)
   	{
   	while(*szBuf) {
      	U1TXREG = *szBuf++;   // transfer data byte to TX reg 
      	while(!U1STAbits.TRMT);  // wait for xmit to complete
      	}	    
   	}

void dbg_PrintAltitude(s16 alt)
	{
    sprintf(gszBuf,"%d m\r\n",alt);
    uart_Print(gszBuf);
	}


void dbg_PrintClimbRate(s16 cps)
	{
    sprintf(gszBuf,"%d cps\r\n",cps);
    uart_Print(gszBuf);
	}

void dbg_PrintTemperature(s16 temp)
	{
    sprintf(gszBuf,"%d C\r\n",temp);
    uart_Print(gszBuf);
	}

#endif // DEBUG
