#include "common.h"
#include "flash.h"

u8 gFlashBuf[FLASH_PAGE_SIZE];

#define FLASH_PAGE_ADDR  ((u32)0x9C00)  

void flash_Init(void) {
	flash_ReadPage();
	if (gFlashBuf[FLASH_PAGE_SIZE-1] == 0xFF) {
		memset(gFlashBuf,0,FLASH_PAGE_SIZE);
		flash_ErasePage();
		flash_WritePage();
		}
	}   
	
void flash_ReadPage(void) {
	uReg32 fAddr;
	fAddr.Val32 = FLASH_PAGE_ADDR;
    ReadPM(gFlashBuf, fAddr);
	}

void flash_ErasePage(void) {
	uReg32 fAddr;
	fAddr.Val32 = FLASH_PAGE_ADDR;
    Erase(fAddr.Word.HW,fAddr.Word.LW,PM_ROW_ERASE);
	fAddr.Val32 = FLASH_PAGE_ADDR+512L;
    Erase(fAddr.Word.HW,fAddr.Word.LW,PM_ROW_ERASE);
	fAddr.Val32 = FLASH_PAGE_ADDR+1024L;
    Erase(fAddr.Word.HW,fAddr.Word.LW,PM_ROW_ERASE);
	}

void flash_WritePage(void) {
	uReg32 fAddr;
	fAddr.Val32 = FLASH_PAGE_ADDR;
    WritePM(gFlashBuf, fAddr);  
	}


void ReadPM(u8 * ptrData, uReg32 SourceAddr) {
  	int    Size;
  	uReg32 Temp;

  	for (Size = 0; Size < PM_ROW_SIZE; Size++) {
    	Temp.Val32 = ReadLatch(SourceAddr.Word.HW, SourceAddr.Word.LW);
    	ptrData[0] = Temp.Val[2];;
    	ptrData[1] = Temp.Val[1];;
    	ptrData[2] = Temp.Val[0];;
    	ptrData = ptrData + 3;
    	SourceAddr.Val32 = SourceAddr.Val32 + 2;
  		}
	}


void WritePM(u8 * ptrData, uReg32 SourceAddr) {
  	int    Size,Size1;
  	uReg32 Temp;
  	uReg32 TempAddr;
  	uReg32 TempData;

  	for (Size = 0,Size1=0; Size < PM_ROW_SIZE; Size++) {
    	Temp.Val[0] = ptrData[Size1+2];
	    Temp.Val[1] = ptrData[Size1+1];
	    Temp.Val[2] = ptrData[Size1+0];	    
		Temp.Val[3] = 0;
	    Size1 += 3;
	
	    WriteLatch(SourceAddr.Word.HW, SourceAddr.Word.LW,Temp.Word.HW,Temp.Word.LW);
	
    	// Device ID errata workaround: Save data at any address that has LSB 0x18 
    	if ((SourceAddr.Val32 & 0x0000001F) == 0x18) {
      		TempAddr.Val32 = SourceAddr.Val32;
      		TempData.Val32 = Temp.Val32;
    		}

    	if ((Size != 0) && (((Size + 1) % 64) == 0)) {
      		// Device ID errata workaround: Reload data at address with LSB of 0x18 
      		WriteLatch(TempAddr.Word.HW, TempAddr.Word.LW,TempData.Word.HW,TempData.Word.LW);
      		WriteMem(PM_ROW_WRITE);
    		}

    	SourceAddr.Val32 = SourceAddr.Val32 + 2;
  		}
	}


//write these in C so that can use the
//__PIC24H__, __PIC24F__ defines.

void WriteMem(u16 val) {
  	asm("mov	W0,NVMCON");
  	__builtin_write_NVM();

  	asm("1:	btsc NVMCON,#15");    //	;Wait for write end
  	asm("	bra 1b");
	}

//_LoadAddr:	;W0=NVMADRU,W1=NVMADR - no return values
void LoadAddr(u16 nvmadru, u16 nvmadr) {
  	asm("mov	W0,TBLPAG");
  	asm("mov	W1,W1");
	}

//_WriteLatch: ;W0=TBLPAG,W1=Wn,W2=WordHi,W3=WordLo - no return values
void WriteLatch(u16 addrhi,u16 addrlo, u16 wordhi, u16 wordlo) {
  	asm("	mov	W0,TBLPAG");
  	asm("	tblwtl W3,[W1]");
  	asm("	tblwth W2,[W1]");
	}

//_ReadLatch: ;W0=TBLPAG,W1=Wn - data in W1:W0
u32 ReadLatch(u16 addrhi, u16 addrlo) {
  	asm("	mov	W0,TBLPAG");
  	asm("	tblrdl [W1],W0");
  	asm("	tblrdh [W1],W1");
	}

void ResetDevice(void) {
  	asm(" goto 0xc02");
	}

//_Erase:
void Erase(u16 addrhi, u16 addrlo, u16 val ) {
  	asm("push    TBLPAG");
  	asm("mov	W2,NVMCON");
  	asm("mov     w0,TBLPAG");           //; Init Pointer to page to be erased
  	asm("tblwtl  w1,[w1]");		        //; Dummy write to select the row

  	__builtin_write_NVM();

  	asm("1:	btsc NVMCON,#15");    //	;Wait for write end
  	asm("	bra 1b");
  	asm("pop     TBLPAG");
	}


