#include "common.h"
#include "i2c.h"

#define I2C_CLK_STRETCH() 	{Nop();}
#define SET_SDA_INPUT() 	{_TRISB8 = 1; ENABLE_RB8_PULLUP(); Nop();}
#define SET_SDA_OUTPUT() 	{CONFIG_RB8_AS_DIG_OUTPUT(); Nop();}
#define SET_SCL_OUTPUT() 	{CONFIG_RB9_AS_DIG_OUTPUT();Nop();}

int gi2c_error;

void i2c_Init(void)	{
    SET_SCL_OUTPUT();
    SET_SDA_OUTPUT();
    I2C_CLK_STRETCH();
    SDA = 1;
    SCL = 1;
    I2C_CLK_STRETCH();
    gi2c_error = 0;
	}


int i2c_Start(void)    {
    SET_SDA_OUTPUT();
    SDA = 1;
    SCL = 1; 
    I2C_CLK_STRETCH();
    I2C_CLK_STRETCH();
    SDA = 0;  // Pull SDA down while SCL is high ... start condition
    I2C_CLK_STRETCH();
    I2C_CLK_STRETCH();
    SCL = 0; //  Pull SCL low for next bit
    I2C_CLK_STRETCH();
    return 0;
    }

int i2c_Stop(void)    {
    SET_SDA_OUTPUT();
    I2C_CLK_STRETCH();
    SDA = 0;  
    I2C_CLK_STRETCH();
    SCL = 1;  
    I2C_CLK_STRETCH();
    SDA = 1;  
    I2C_CLK_STRETCH();
    return 0;
    }

int i2c_XBit(int bit)   {
   SET_SDA_OUTPUT();
   I2C_CLK_STRETCH();
   SDA = (bit ? 1 : 0);
   I2C_CLK_STRETCH();
   SCL = 1;
   I2C_CLK_STRETCH();
   SCL = 0;
   I2C_CLK_STRETCH();
   return 0;
   }

 
int i2c_RBit(void)   {
   int bit;
   SET_SDA_INPUT();
   I2C_CLK_STRETCH();
   SCL = 1;
   I2C_CLK_STRETCH();
   bit = (SDA ? 1 : 0);  
   SCL = 0;
   I2C_CLK_STRETCH();
   return bit;
   }

int i2c_XByte(u8 dat)    {
    int bitCount = 8;
    while(bitCount--)        {
        i2c_XBit((dat & 0x80)? 1 : 0);
        dat <<= 1;
        }

    if (i2c_RBit())        {
        gi2c_error = I2CERR_NAK;
        return 1;
        }
    return 0;
    }

u8 i2c_RByte(int ack)    {
    u8 inByte = 0;
    int bitCount = 8;

    while(bitCount--)        {
        inByte <<= 1;
        inByte |= i2c_RBit();
        }

    // acknowledge
    i2c_XBit(ack);
    return inByte;
    }


int i2c_XmtBuf(u8 devId, u8 addr, int nBytes, u8* pBuf )    {
    int cnt;

    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))        {
        return 0xF0;
        }
    if (i2c_XByte(addr))        {
        return 0xF1;
        }
    for (cnt = 0; cnt < nBytes; cnt++)
        if (i2c_XByte(pBuf[cnt]))            {
            return 0xF2;
            }
    i2c_Stop();
    return 0;
    }




int i2c_RcvBuf(u8 devId, u8 addr, int nBytes, u8* pBuf)    {
    int cnt;

    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))        {
        return 0xF0;
        }
    if (i2c_XByte(addr))        {
        if (gi2c_error == I2CERR_NAK) i2c_Stop();
        return 0xF1;
        }
    SET_SDA_OUTPUT();
    SDA = 1; // Set SDA high while SCL is low
    I2C_CLK_STRETCH();
    // Restart
    i2c_Start();
    if (i2c_XByte(devId|0x1))        {
        return 0xF2;
        }

    for (cnt = 0; cnt < nBytes-1; cnt++)        {
        pBuf[cnt] = i2c_RByte(0);
        }
    // Master NAK
    pBuf[nBytes-1] = i2c_RByte(1);
    i2c_Stop();
    return (gi2c_error ? 1 : 0);
    }



int i2c_XmtByte(u8 devId, u8 addr, u8 data)    {
    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))        {
        return 0xF0;
        }
    if (i2c_XByte(addr))        {
        return 0xF1;
        }
    if (i2c_XByte(data))        {
        return 0xF2;
        }
    i2c_Stop();
    return 0;
    }


u8 i2c_RcvByte(u8 devId, u8 addr)    {
    u8 data;
    
    gi2c_error = 0;
    i2c_Start();
    if (i2c_XByte(devId))        {
        return 0xF0;
        }
    if (i2c_XByte(addr))        {
        if (gi2c_error == I2CERR_NAK) i2c_Stop();
        return 0xF1;
        }
	SET_SDA_OUTPUT();
    SDA = 1; // Set SDA high while SCL is low
    // Restart
    I2C_CLK_STRETCH();
    i2c_Start();
    if (i2c_XByte(devId|0x1))        {
        return 0xF2;
        }
    data = i2c_RByte(1);
    i2c_Stop();
    return data;
    }
