#include "common.h"
#include "nvd.h"
#include "i2c.h"
#include "lcd.h"
#include "tmr.h"
#include "flash.h" 
#include "util.h" 
#include "ui.h" 
#include "uart.h"

s16 gnSinkToneCps;
s16 gnClimbToneCps;
u8 gnFLogOldestRec; // memory  index
s16 gnAltCorrM;
u8 gnFlightTrigM;

u8* gNvdParBuf;
u8 gbNvdChange;

void nvd_Init(void)   {
   	u8 cksum;
	flash_Init();
	gbNvdChange = 0;
	gNvdParBuf = gFlashBuf + NVD_PAR_OFFSET;
   	cksum = ~(gNvdParBuf[0]+gNvdParBuf[1]+gNvdParBuf[2]+gNvdParBuf[3]+gNvdParBuf[4]+gNvdParBuf[5]);
   	if (cksum == gNvdParBuf[NVD_ADDR_CKSUM])      {
		gnAltCorrM = (s16)(((u16)gNvdParBuf[NVD_ADDR_ALTCORR_L])+(((u16)gNvdParBuf[NVD_ADDR_ALTCORR_M])<<8));
		gnSinkToneCps = -10*(s16)gNvdParBuf[NVD_ADDR_SINKTONE_CPS];
		gnClimbToneCps = ((s16)gNvdParBuf[NVD_ADDR_CLIMBTONE_CPS])+CLIMBTONE_CPS_MIN;
	    gnFlightTrigM = gNvdParBuf[NVD_ADDR_FLIGHT_TRIG];
		gnFLogOldestRec = gNvdParBuf[NVD_ADDR_FLOG_OLDEST_REC];
		}
   else  {
		lcd_ClearScreen();
		lcd_Printf(0,0,"MEMORY CORRUPT");
		lcd_Printf(1,0,"PRESS (C) ");
		lcd_Printf(2,0,"   TO CLEAR");
#ifdef UART_DBG
		uart_Print("Memory corrupt, press C to clear\r\n");
#endif
		while(!gbBtnCPressed);
		ui_BtnClear();
		lcd_Printf(3,0,"...");
		memset(gFlashBuf,0,FLASH_PAGE_SIZE);
		gnAltCorrM = 0; 
		nvd_WriteAltCorr();
		gnSinkToneCps = SINKTONE_CPS_DEF;
		nvd_WriteSinkToneCps();
		gnClimbToneCps = CLIMBTONE_CPS_DEF;
		nvd_WriteClimbToneCps();
		gnFlightTrigM = FLIGHT_TRIG_DEF;
        nvd_WriteFlightTrig();
		gnFLogOldestRec = 0;
		nvd_WriteFLogOldestRec();
		nvd_UpdateFlash();
		lcd_Printf(4,0,"DONE!");
#ifdef UART_DBG
		uart_Print("Done!\r\n");
#endif
		tmr1_BusyWaitMs(1500);
		}
   }

void nvd_UpdateCksum(void)   {
	u8 cksum = ~(gNvdParBuf[0]+gNvdParBuf[1]+gNvdParBuf[2]+gNvdParBuf[3]+gNvdParBuf[4]+gNvdParBuf[5]);
	gNvdParBuf[NVD_ADDR_CKSUM] = cksum;
	gbNvdChange = 1;
	}

void nvd_WriteAltCorr(void)   {
	gNvdParBuf[NVD_ADDR_ALTCORR_L] = ((u16)gnAltCorrM)&0xFF;
	gNvdParBuf[NVD_ADDR_ALTCORR_M] = (((u16)gnAltCorrM)>>8)&0xFF;
	nvd_UpdateCksum();
	}

void nvd_WriteFlightTrig(void)   {
	gNvdParBuf[NVD_ADDR_FLIGHT_TRIG] = gnFlightTrigM;
	nvd_UpdateCksum();
	}

void nvd_WriteSinkToneCps(void)   {
	gNvdParBuf[NVD_ADDR_SINKTONE_CPS] = (u8)(-gnSinkToneCps/10);
	nvd_UpdateCksum();
	}

void nvd_WriteClimbToneCps(void)   {
	gNvdParBuf[NVD_ADDR_CLIMBTONE_CPS] = (u8)(gnClimbToneCps-CLIMBTONE_CPS_MIN);
	nvd_UpdateCksum();
	}

void nvd_WriteFLogOldestRec(void)   {
	gNvdParBuf[NVD_ADDR_FLOG_OLDEST_REC] = gnFLogOldestRec;
	nvd_UpdateCksum();
	gbNvdChange = 1;
	}


void nvd_UpdateFlash(void) {
	if (gbNvdChange) {
		flash_ErasePage();
		flash_WritePage();
		gbNvdChange = 0;
		}
	}

