#include "common.h"
#include "sensor.h"
#include "tmr.h"
#include "i2c.h"
#include "audio.h"
#include "ui.h"
#include "nvd.h"
#ifdef UART_DBG
#include "uart.h"
#endif

s16 gAC1;
s16 gAC2;
s16 gAC3;
u16 gAC4;
u16 gAC5;
u16 gAC6;
s16 gB1;
s16 gB2;
s32 gB5;
s16 gMb;
s16 gMc;
s16 gMd;

int gnSensorState;
int gnSmpCnt;
s32 gZBuf[SNS_NUM_Z_SAMPLES];
s32 gnAltAvgM; 
s32 gnAltAvgCm;
s32 gnAltCm;
s32 gnCps;
u32 gnTraw;
u32 gnPraw;
s16 gnTempC;
s32 gnSum_t;
s32 gnSum_t2;
s32 gnDen;
s32 gnPa;





void sns_Init(void)   {
	gnSmpCnt = 0;
	gnAltAvgM = 0;
	gnAltCm = 0;
	gnCps = 0;
	gnTempC = 0;
	gnPa = 0;
	// following parameters used in linear regression computation
    // for climb and sink rate, need to be computed only once.
	gnSum_t = -((s32)SNS_NUM_Z_SAMPLES * (SNS_NUM_Z_SAMPLES-1))/2;
	gnSum_t2 = ((s32)SNS_NUM_Z_SAMPLES * (SNS_NUM_Z_SAMPLES-1)*(2*SNS_NUM_Z_SAMPLES-1))/6;
	gnDen = ((s32)SNS_NUM_Z_SAMPLES*gnSum_t2) - (gnSum_t*gnSum_t);
	sns_ReadCoeffs();
#ifdef UART_DBG
	#ifdef SENSOR_UNIT_TEST
	sns_UnitTest();
	#endif
#endif
   	}


void sns_ReadCoeffs(void){	
	u8 b[22];
	i2c_RcvBuf(I2C_ID_BMP085,0xAA,22,b);

	gAC1 = (s16)((((u16)b[0])<<8) | (u16)b[1]);
	gAC2 = (s16)((((u16)b[2])<<8) | (u16)b[3]);
	gAC3 = (s16)((((u16)b[4])<<8) | (u16)b[5]);
	
	gAC4 = (((u16)b[6])<<8) | (u16)b[7];
	gAC5 = (((u16)b[8])<<8) | (u16)b[9];
	gAC6 = (((u16)b[10])<<8) | (u16)b[11];

	gB1 = (s16)((((u16)b[12])<<8) | (u16)b[13]);
	gB2 = (s16)((((u16)b[14])<<8) | (u16)b[15]);
	gMb = 0;
	gMc = (s16)((((u16)b[18])<<8) | (u16)b[19]);
	gMd = (s16)((((u16)b[20])<<8) | (u16)b[21]);
#ifdef UART_DBG
  	sprintf(gszBuf,"AC1 %d\r\nAC2 %d\r\nAC3 %d\r\nAC4 %u\r\nAC5 %u\r\nAC6 %u\r\n",gAC1, gAC2, gAC3, gAC4, gAC5, gAC6);
    uart_Print(gszBuf);

  	sprintf(gszBuf,"B1 %d\r\nB2 %d\r\n",gB1, gB2);
    uart_Print(gszBuf);

  	sprintf(gszBuf,"MB %d\r\nMC %d\r\nMD %d\r\n",gMb, gMc, gMd);
    uart_Print(gszBuf);
#endif
/*
AC1 11663
AC2 -826
AC3 -14324
AC4 32514
AC5 25609
AC6 16273
B1 5498
B2 38
MB 0
MC -11075
MD 2432
*/
	}

void sns_InitData(void) {
   sns_InitWindowBuffers();
   gnCps = 0;
   gnSmpCnt = SNS_NUM_Z_SAMPLES-1;

   sns_TriggerTemperatureSample();
   gnSensorState = SENSOR_READ_TEMPERATURE;
   }


#ifdef SENSOR_UNIT_TEST

// Bosch SensorTec test to validate the code used to compute temperature
// and pressure from pressure sensor readings 

u16 ac5[9] = {32758, 32758, 32758, 30774, 30774, 30774, 26431, 26431, 26431};
u16 ac6[9] = {23153, 23153, 23153, 9469, 9469, 9469, 40099, 40099, 40099};

u32 up[9] = {25827, 23843, 21476, 25634, 23656, 21302, 26060, 24056, 21657};
u32 ut[9] = {24524, 27898, 34787, 9389, 14520, 21853, 40438, 45980, 54518};

s32  pResult[9]= {70369, 69964, 69961, 66140, 69406, 69385, 68346, 70599, 70559 };
s16  tResult[9] = {-177, 150, 650, -404, 150, 650, -338, 150, 650};

void sns_UnitTest(void){
	int cnt;
	s16 t;
	s32 p;

    gAC1 = 408;
    gAC2 = -72;
    gAC3 = -14383;
    gAC4 = 32741;
    gB1 = 6190;
    gB2 = 4;
	gMb = 0;
	gMc = -8711;
	gMd = 2868;
	for (cnt = 0; cnt < 9; cnt++){
		gAC5 = ac5[cnt];
		gAC6 = ac6[cnt];
		gnTraw = ut[cnt];
		gnPraw = up[cnt];
		t = sns_CalcTemperatureCx10();
		p = sns_CalcPressurePa();
        sprintf(gszBuf,"%d T %d %d P %ld %ld\r\n",cnt, tResult[cnt],t, pResult[cnt], p);
		uart_Print(gszBuf);
		}
}
#endif


void sns_SampleContinuous(void) {
   s16 t;
   static s32 pBuf[3];
   static s16 pCnt = 0;
   static u16 gnDispTick = 0;

   if (!gpioSnsEoc) return;
   if (gnSensorState == SENSOR_READ_TEMPERATURE) {
      gnTraw = sns_ReadTemperatureSample();
      sns_TriggerPressureSample();
      gpioDbg = 1;
      t = sns_CalcTemperatureCx10();
      gnTempC = (t >= 0?  (t+5)/10 : (t-5)/10);
      pBuf[pCnt++] = sns_CalcPressurePa();
      if (pCnt == 3) {
		 pCnt = 0;
         gnPa = (pBuf[0]+pBuf[1]+pBuf[2]+1)/3;
      	 gnPa = ((gnPa+4)>>3);
      	 gZBuf[gnSmpCnt] = gnAltCm = sns_Pa2Cm(gnPa<<3);
      	 sns_AverageAltitude();
      	 sns_DeltaCmPerSec();
		 //sns_LPFClimbRate();
      	 gnSmpCnt++;
      	 if (gnSmpCnt == SNS_NUM_Z_SAMPLES) {
         	 gnSmpCnt = 0;
         	 }
      	 if (gnAudioMode != AUDIO_MODE_OFF) {
         	audio_Beep(gnCps);
         	}
		 }
      	   if (gbTimerEn)  {
         	  ui_UpdateFlightData();
         	  }     

	   gnDispTick++;
	   switch (gnDispTick&7) {
		   case 0 : ui_PrintAltitude(0,0,(s16)gnAltAvgM); 
				break;
		   case 1 : ui_PrintClimbRate(3,48,gnCps); 
				break;
		   case 2 : ui_PrintAltitudeAL(3,0,(s16)gnAltAvgM,(s16)gnLaunchAltM);
				break;
		   case 3 : ui_PrintTemperature(1,66,gnTempC); 
				break;
		   case 4 : ui_PrintClimbRate(3,48,gnCps); 
				break;
		   case 5 : ui_PrintRealTime(5,0,gnRealHrs,gnRealMin); 
				break;
		   case 6 : ui_PrintFlightTime(5,61,gnTimerHrs,gnTimerMin);
				break;
		   default  :
		   case 7: ui_PrintClimbRate(3,48,gnCps); 
				break;
		   }

	  gpioDbg = 0;
	  gnSensorState = SENSOR_READ_PRESSURE;
      }
   else
   if (gnSensorState == SENSOR_READ_PRESSURE) {
      gnPraw = sns_ReadPressureSample();
      sns_TriggerTemperatureSample();
      gnSensorState = SENSOR_READ_TEMPERATURE;
      }
   }




void sns_InitWindowBuffers()   {
   int n;
   sns_AcquireAveragedSample(4);
   n = SNS_NUM_Z_SAMPLES;
   while(n--) {
      gZBuf[n] = gnAltCm;
      }
   }


void sns_TriggerPressureSample(void)   {
   i2c_XmtByte(I2C_ID_BMP085,0xF4,0x34+(SENSOR_OSS<<6));
   }

void sns_TriggerTemperatureSample(void)   {
   i2c_XmtByte(I2C_ID_BMP085,0xF4,0x2E);
   }

void sns_AcquireAveragedSample(int nSamples) {
   s32 pa,tc,pAccum,tAccum,n;
   pAccum = 0;
   tAccum = 0;
   n = nSamples;
   while (n--) {
      sns_TriggerTemperatureSample();
      while(!gpioSnsEoc);
      gnTraw = sns_ReadTemperatureSample();
      tc = sns_CalcTemperatureCx10();
      sns_TriggerPressureSample();
      while(!gpioSnsEoc);
      gnPraw = sns_ReadPressureSample();
      pa = sns_CalcPressurePa();
      pAccum += pa;
      tAccum += tc;
      }
   tc = tAccum/nSamples;
   gnTempC = (tc >= 0 ?  (tc+5)/10 : (tc-5)/10);
   pa = pAccum/nSamples;
   gnAltCm = sns_Pa2Cm(pa);
   gnAltAvgM = (gnAltCm >= 0 ? (gnAltCm+50L)/100L : (gnAltCm-50L)/100L);
   gnAltAvgM += gnAltCorrM;
#ifdef UART_DBG
   sprintf(gszBuf,"Tavg : %dC\r\n",gnTempC);
   uart_Print(gszBuf);
   sprintf(gszBuf,"Pavg : %ldpa\r\n",pa);
   uart_Print(gszBuf);
   sprintf(gszBuf,"Zavg : %ldm\r\n",gnAltAvgM);
   uart_Print(gszBuf);
#endif
   }


#ifdef SENSOR_NOISE_EVAL

void sns_DumpAltitude(int nSamples, int pzInx)   {
   	s32 pa,tc,n;
   	n = nSamples;
   	while (n--)  {
		sns_TriggerTemperatureSample();
        while(!gpioSnsEoc);
		gnTraw = sns_ReadTemperatureSample();
		tc = sns_CalcTemperatureCx10();
		sns_TriggerPressureSample();
        while(!gpioSnsEoc);
		gnPraw = sns_ReadPressureSample();
		pa = sns_CalcPressurePa();
   		gnTempC = (tc >= 0 ?  (tc+5)/10 : (tc-5)/10);
   		gnAltCm = sns_Pa2Cm(pa, pzInx);
#ifdef UART_DBG
   		//sprintf(gszBuf,"%ldpa\r\n",pa);
   		//uart_Print(gszBuf);
   		sprintf(gszBuf,"%ld\r\n",gnAltCm);
   		uart_Print(gszBuf);
		}
#endif
   }
#endif


u32 sns_ReadTemperatureSample(void)	{
	u8 b[2];
	u32 w;
    i2c_RcvBuf(I2C_ID_BMP085,0xF6,2,b);
    w = (((u32)b[0])<<8) | (u32)b[1];
	return w;
	}

u32 sns_ReadPressureSample(void)	{
	u8 b[3];
	u32 dw;
    i2c_RcvBuf(I2C_ID_BMP085,0xF6,3,b);
    dw = (((u32)b[0])<<16) | (((u32)b[1])<<8) | (u32)b[2];
    dw = (dw >> (8-SENSOR_OSS));
    return dw;
    }


s16 sns_CalcTemperatureCx10(void)	{
	s32 x1,x2;
	s16 t;
    x1 = (((s32) gnTraw - (s32) gAC6) * (s32) gAC5) >> 15;
    x2 = ((s32) gMc << 11) / (x1 + gMd);
    gB5 = x1 + x2;
    t = ((gB5+8)>>4);
	return t;
	}

s32 sns_CalcPressurePa(void)	{
	s32 x1,x2,x3,b3,b6,p;
	u32 b4,b7;

	b6 = gB5 - 4000L;
	x1 = (b6*b6) >> 12;	 	 
	x1 *= gB2;
	x1 >>= 11;

	x2 = (gAC2*b6);
	x2 >>= 11;
	x3 = x1 +x2;
	b3 = (((((s32)gAC1 )*4 + x3) << SENSOR_OSS) + 2) >> 2;

	x1 = (gAC3* b6) >> 13;
	x2 = (gB1 * ((b6*b6) >> 12) ) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (gAC4 * (u32) (x3 + 32768L)) >> 15;
     
	b7 = ((u32)(gnPraw - b3) * (50000L >> SENSOR_OSS));   
	if (b7 < (u32)0x80000000) {
		p = (b7 << 1) / b4;
		}
	else { 
		p = (b7 / b4) << 1;
		}
   
	x1 = p >> 8;
	x1 *= x1;
	x1 = (x1 * 3038L) >> 16;
	x2 = (p * -7357L) >> 16;
	p += ((x1 + x2 + 3791L) >> 4);	// p in Pa  
    return p;
	}
	


void sns_AverageAltitude(void)   {
   int cnt;
   s32 avg;
   cnt = SNS_NUM_Z_SAMPLES;
   avg = 0;
   while (cnt--)  {
		avg += gZBuf[cnt];
		}
   avg /= SNS_NUM_Z_SAMPLES;
   gnAltAvgCm = avg;
   gnAltAvgM = ((avg >= 0) ? (avg+50L)/100L : (avg-50L)/100L); 
   gnAltAvgM += gnAltCorrM;
   return; 
   }

void sns_DeltaCmPerSec(void)   {
   	int n,trel;
   	s32 z, sum_zt;
   	
   	sum_zt = 0;
   	n = SNS_NUM_Z_SAMPLES;
   	while (n--)  {
      	z = gZBuf[n] - gnAltAvgCm;
      	trel = n - gnSmpCnt;// time origin is the current sample in window
      	if (n > gnSmpCnt) {
			trel -= SNS_NUM_Z_SAMPLES;
			}
      	sum_zt += ((s32)trel*z);
      	}
    gnCps = (sum_zt*(s32)(SENSOR_SAMPLES_PER_SECX10*SNS_NUM_Z_SAMPLES))/(10*gnDen);
   	return;
   	}

#include "pztbl.txt"

s32 sns_Pa2Cm(s32 pa)  {
   	s32 inx,pa1,z1,z2,z;
   	if (pa > PA_INIT) {
      	z = gPZTbl[0];
      	}
   	else {
      	inx = (PA_INIT - pa)>>10;      
      	if (inx >= PZLUT_ENTRIES-1) {
         	z = gPZTbl[PZLUT_ENTRIES-1];
         	}
      	else {
         	pa1 = PA_INIT - (inx<<10);
         	z1 = gPZTbl[inx];
         	z2 = gPZTbl[inx+1];
         	z = z1 + (((pa1-pa)*(z2-z1))>>10);
         	}
      	}
   	return z;
   	}



