#include "common.h"
#include "tmr.h"

volatile u32 gBtnLState = 0;
volatile u32 gBtnRState = 0;
volatile u32 gBtnUState = 0;
volatile u32 gBtnDState = 0;
volatile u8 gBtnCState = 0;
volatile u8 gbBtnLPressed = 0;
volatile u8 gbBtnRPressed = 0;
volatile u8 gbBtnUPressed = 0;
volatile u8 gbBtnDPressed = 0;
volatile u8 gbBtnCPressed = 0;

volatile u8 gnTimerHrs;
volatile u8 gnTimerMin;
volatile u8 gnTimerSec;

volatile u32 gnTimeTick;
volatile u16 gnSecTick;

volatile u8  gnRealHrs;
volatile u8  gnRealMin;
volatile u8  gnRealSec;

volatile u8 gbTmr4Flag;
volatile u8 gbTmr3Flag;

u8 gbTimerEn;

// TMR1 functions used for busy waits, code profiling
void  tmr1_Config(void) {
   T1CONbits.TON = 0;    // turn off the timer
   T1CONbits.TCS = 0;    // select internal clock 
   T1CONbits.TGATE = 0;  // gate disabled
   T1CONbits.TSIDL = 0;  // continue in idle mode
   T1CONbits.TCKPS1 = 1; // Prescaler 1:256
   T1CONbits.TCKPS0 = 1; // Prescaler 1:256

   PR1 = 65535;
   TMR1  = 0;            //clear timer1 value
   _T1IF = 0;            //clear interrupt flag
   _T1IP = 1;            //choose a priority
   _T1IE = 0;            //disable the interrupt
   }


void tmr1_Start(void) {
   	T1CONbits.TON = 0; //turn off the timer
   	TMR1  = 0;         //zero timer1 counter
   	T1CONbits.TON = 1; //turn on the timer
	}

int tmr1_ElapsedTimeMs(void) {
    int ms;
   	T1CONbits.TON = 0; //turn off the timer
    ms = (int)(((TMR1*1000L)<<8)/FOSC_HZ);
    return ms;
	}

// max delay is 4194 mS @ 4MHz and prescaler 256
void tmr1_BusyWaitMs(int mS) {
    u16 cnt = (u16)(((mS * FOSC_HZ)/1000L)>>8);
   	T1CONbits.TON = 0; // turn off timer
   	TMR1  = 0;         // zero timer1 counter
   	T1CONbits.TON = 1; // turn on timer
    while(TMR1 < cnt); // busy wait until interval completes
    T1CONbits.TON = 0;
    return;
	}

// TMR2 used for generating vario audio tone square wave 
void tmr2_Config(void){
    T2CONbits.T32 = 0; // 16bit timer
    T2CONbits.TCS = 0; //  select internal clock 
    T2CONbits.TGATE = 0; // gate disabled
    T2CONbits.TSIDL = 0; // continue in idle mode
    T2CONbits.TCKPS1 = 0; // Prescaler 1:8
    T2CONbits.TCKPS0 = 1; // Prescaler 1:8
	}

// TMR3 used for button sampling and 50mS clock time tick
#define FREQ_TMR3_HZ   20L

void tmr3_Config( void){
	_T3IE = 0;  // disable interrupt
	T2CONbits.T32 = 0; // 16bit timer
    T3CONbits.TON = 0; // 16bit timer
    T3CONbits.TCS = 0; //  select internal clock 
    T3CONbits.TGATE = 0; // gate disabled
    T3CONbits.TSIDL = 0; // continue in idle mode
    T3CONbits.TCKPS1 = 0; // Prescaler 1:8
    T3CONbits.TCKPS0 = 1; // Prescaler 1:8
    PR3 = (u16)((FOSC_HZ>>3)/FREQ_TMR3_HZ );
    TMR3  = 0;                       //zero timer3 counter
    _T3IF = 0;                       //clear interrupt flag
    _T3IP = 1;                       //choose a priority
	_T3IE = 1;  // enable interrupt
    T3CONbits.TON = 1; // start tmr3
	}

void __attribute__((interrupt, no_auto_psv)) _T3Interrupt(void) {
  	_T3IF = 0;    //clear the timer interrupt bit

   gbTmr3Flag = 1;
   gBtnLState = (gBtnLState<<1) | (u32)gpioBtnL;
   if ((gBtnLState&15UL) == 8)      {
      gbBtnLPressed = 1;
      }
   gBtnRState = (gBtnRState<<1) | (u32)gpioBtnR;
   if ((gBtnRState&15UL) == 8)      {
      gbBtnRPressed = 1;
      }
   gBtnUState = (gBtnUState<<1) | (u32)gpioBtnU;
   if ((gBtnUState&15UL) == 8)      {
      gbBtnUPressed = 1;
      }
   gBtnDState = (gBtnDState<<1) | (u32)gpioBtnD;
   if ((gBtnDState&15UL) == 8)      {
      gbBtnDPressed = 1;
      }
   gBtnCState = (gBtnCState<<1) | (u8)gpioBtnC;
   if ((gBtnCState&0x0f) == 8)      {
      gbBtnCPressed = 1;
      }

   gnTimeTick++;
   gnSecTick++;
   if (gnSecTick == FREQ_TMR3_HZ)  {
      gnSecTick = 0;
      gnRealSec++;
      if (gnRealSec == 60) {
         gnRealSec = 0;
         gnRealMin++;
         if (gnRealMin == 60)  {
            gnRealMin = 0;
            gnRealHrs++;
            if (gnRealHrs == 24)
               gnRealHrs = 0;
            }
         }
      
      if (gbTimerEn)  {
         gnTimerSec++;
         if (gnTimerSec == 60)  {
            gnTimerSec = 0;
            gnTimerMin++;
            if (gnTimerMin == 60)  {
               gnTimerMin = 0;
               gnTimerHrs++;
               }
            }
         }
      }
   }
	


// TMR4 used for immediate return, one-shot interval timing
// gbTmr4Flag is set at end of timed interval.

void tmr4_Config(void){
    _T4IE = 0;         		// disable timer interrupt
   	T4CONbits.TON = 0; 		// turn off the timer
	T4CONbits.T32 = 0; 		// 16bit timer
    T4CONbits.TCS = 0; 		// select internal clock 
    T4CONbits.TGATE = 0; 	// gate disabled
    T4CONbits.TSIDL = 0; 	// continue in idle mode
    T4CONbits.TCKPS1 = 0; 	// Prescaler 1:8
    T4CONbits.TCKPS0 = 1; 	// Prescaler 1:8
    PR4 = 65535;
    TMR4  = 0;              //zero timer4 counter
    _T4IF = 0;              //clear interrupt flag
    _T4IP = 1;              //choose a priority
	}

void tmr4_Load(u16 mS) {
    _T4IE = 0;
   	T4CONbits.TON = 0; // turn off timer
    PR4 = ((mS * FOSC_HZ)>>3) / 1000L;
	}
	
void tmr4_Start(void) {
   	T4CONbits.TON = 0; // turn off timer
   	TMR4  = 0;         // zero timer4 counter
    _T4IE = 1;
   	T4CONbits.TON = 1; // turn on  timer
	}

void __attribute__((interrupt, no_auto_psv)) _T4Interrupt(void) {
  	_T4IF = 0;    // clear timer interrupt bit
    gbTmr4Flag = 1;
	}

