#include "common.h"
#include "ui.h"
#include "lcd.h"
#include "sensor.h"
#include "nvd.h"
#include "rtc.h"
#include "tmr.h"
#include "util.h"
#include "audio.h"
#include "flash.h" 
#ifdef UART_DBG
#include "uart.h"
#endif

u8 gbStable = 0; 

u8 gnUIMode;
u8 gnParSel;

s16 gnMaxAltM;
s16 gnMaxCps;
s16 gnMinCps;
s16 gnMaxTempC;
s16 gnMinTempC;
s16 gnLaunchAltM;
s16 gnLZAltM;
u8  gnLaunchHrs;
u8  gnLaunchMin;

const  char* szMon[] = {"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG","SEP","OCT","NOV","DEC"};

u8 gnFLogRec;    // display index
u8* gpFLogBuf;

void ui_PrintAltitude(u8 row, u8 col, s16 nAlt) {
    s8 cnt;
	char szBuf[5];
    CLAMP(nAlt,MIN_ALTITUDE_M,MAX_ALTITUDE_M);
	szBuf[4] = '\0';
    if (nAlt == 0){
		szBuf[3] = '0';
		szBuf[2] = ' ';
		szBuf[1] = ' ';
		szBuf[0] = ' ';
		}
    else
    if (nAlt < 0){
        nAlt = -nAlt;
        cnt = 3;
        while (nAlt) {
			szBuf[cnt--] = (nAlt%10) + 48;
            nAlt = nAlt/10;
			}
       szBuf[cnt--] = '-';
		while(cnt >= 0) {
			szBuf[cnt--] = ' ';
			}
       }
    else {
        cnt = 3;
        while (nAlt) {
			szBuf[cnt--] = (nAlt%10) + 48;
            nAlt = nAlt/10;
			}
		while(cnt >= 0) {
			szBuf[cnt--] = ' ';
			}
       }
	lcd_PrintLSz(row,col,szBuf);
    }



void ui_PrintAltitudeAL(u8 row, u8 col, s16 nAlt, s16 nLaunchAlt)  {
   char sign;
   s16 nAltAL;
   s8 cnt;
   char szBuf[6];

   nAltAL = (nAlt - nLaunchAlt);
   CLAMP(nAltAL,-9999,9999);
   szBuf[5] = '\0';
   if (nAltAL == 0){
		szBuf[4] = '0';
		szBuf[3] = ' ';
		szBuf[2] = ' ';
		szBuf[1] = ' ';
		szBuf[0] = ' ';
   		} 
	else {
    	if (nAltAL < 0){
        	nAltAL = -nAltAL;
        	sign = '-';
			}
		else {
			sign = '+';
			}		
    	cnt = 4;
    	while (nAltAL) {
			szBuf[cnt--] = (nAltAL%10) + 48;
        	nAltAL = nAltAL/10;
			}
    	szBuf[cnt--] = sign;
		while(cnt >= 0) {
			szBuf[cnt--] = ' ';
			}
    	}
	lcd_PrintSz(row,col,szBuf);
	}


void ui_PrintClimbRate(u8 row, u8 col, s16 nCps) {
   	char sign;
   	s16 dec,frac;
	char szBuf[4];

   	CLAMP(nCps, MIN_CLIMBRATE_CPS, MAX_CLIMBRATE_CPS);
	nCps = (nCps < 0 ?  (nCps-5)/10 : (nCps+5)/10);
	if (nCps > 0){
		sign = '+';
		}
	else
	if (nCps < 0){
		sign = '-';
		nCps = -nCps;
		}
	else {
		sign = ' ';	
		}	
   	szBuf[3] = '\0';
	lcd_SetPos(row,col+24);
   	if (sign == ' ') {
		lcd_PutChar('0');
	  	szBuf[2] = '0';
	  	szBuf[1] = ' ';
	  	szBuf[0] = ' ';
      	}
   else {
   		dec = nCps/10;
   		frac = nCps%10;
		lcd_PutChar(frac+48);
		if (dec == 0) {
	  		szBuf[2] = '0';
	  		szBuf[1] = sign;
	  		szBuf[0] = ' ';
			}
		else {
      		szBuf[2] = (dec%10) + 48; 
	  		dec = dec/10;
			if (dec == 0) {
				szBuf[1] = sign;
				szBuf[0] = ' ';
				}
			else {
				szBuf[1] = dec + 48;
				szBuf[0] = sign;
				}
			}
		}
	lcd_PrintSz(row,col,szBuf);
  	}

 
void ui_PrintTemperature(u8 row, u8 col, s16 nTempC) {
	char szBuf[3];
   	CLAMP(nTempC,MIN_TEMP_C,MAX_TEMP_C);
	szBuf[2] = '\0';
    if (nTempC == 0){
		szBuf[1] = '0';
		szBuf[0] = ' ';
		}
    else
    if (nTempC < 0){
		szBuf[1] = (-nTempC) + 48;
		szBuf[0] = '-';
       }
    else {
		szBuf[1] = (nTempC%10) + 48;
        nTempC = nTempC/10;
		szBuf[0] = (nTempC ? nTempC+48 : ' ');
        }
	lcd_PrintSz(row,col,szBuf);
   	}
	

void ui_PrintRealTime(u8 row, u8 col, u8 nHrs, u8 nMin) 	{
	char szBuf[3];
   	CLAMP(nHrs,0,23);
   	CLAMP(nMin,0,59);
	szBuf[2] = '\0';

	lcd_SetPos(row,col+18);
	lcd_PutChar((nMin/10) + 48);
	lcd_PutChar((nMin%10) + 48);

	lcd_SetPos(row,col+30);
   	if (nHrs/12){
		lcd_PutChar('P');
		if (nHrs == 12) {
			szBuf[1] = '2';
            szBuf[0] = '1';
			}
		else {
			nHrs = nHrs - 12;
			szBuf[1] = (nHrs%10) + 48;
			nHrs = nHrs/10;
			szBuf[0] = (nHrs ?	nHrs+48 : ' ');
			}	
      	}
   	else
   	if (nHrs < 10)	{
		lcd_PutChar('A');
		szBuf[1] = nHrs+48;
		szBuf[0] = ' ';
		}
   	else {
		lcd_PutChar('A');
		szBuf[1] = (nHrs%10)+48;
		szBuf[0] = (nHrs/10)+48;
      	}
	lcd_PrintSz(row,col,szBuf);
   	}


void ui_PrintFlightTime(u8 row, u8 col, u8 nHrs, u8 nMin)	{
	if (!gbStable) {
		lcd_Printf(row,col,"TRG ");
		return;
		}
	if (!gbTimerEn) {
		lcd_Printf(row,col,"RDY ");
		return;
		}
   	CLAMP(nHrs,0,9);
   	CLAMP(nMin,0,59);
    lcd_SetPos(row,col);
	lcd_PutChar(nHrs + 48);
	lcd_PutChar(':');
	lcd_PutChar((nMin/10) + 48);
	lcd_PutChar((nMin%10) + 48);
   	}


void ui_SetEventHandler(void)  {
   	if (gbTmr3Flag)  {
      	gbTmr3Flag = 0;
      	ui_DisplaySettings();
   	if (gbBtnCPressed)  {
	  	ui_BtnClear();
		nvd_UpdateFlash();
      	ui_InitRunMode();
      	}
   	else
   	if (gbBtnDPressed || (gBtnDState == 0UL))  {
	  	ui_BtnClear();
      	gnParSel = (gnParSel < SEL_RTC_YAR ? gnParSel+1 : SEL_ALTITUDE);
      	lcd_ClearScreen();
      	ui_DisplaySettings();
      	}
   	else
   	if (gbBtnUPressed || (gBtnUState == 0UL))  {
	  	ui_BtnClear();
      	gnParSel = (gnParSel > SEL_ALTITUDE ? gnParSel-1 : SEL_RTC_YAR);
      	lcd_ClearScreen();
      	ui_DisplaySettings();
      	}
   	else
   	if (gbBtnLPressed || (gBtnLState == 0UL))  {
	  	ui_BtnClear();
      	switch (gnParSel)  {
         	case SEL_ALTITUDE 	:  default : gnAltCorrM--; if (gnAltCorrM < ALT_CORR_MIN) gnAltCorrM = ALT_CORR_MIN; nvd_WriteAltCorr(); nvd_UpdateFlash(); sns_AcquireAveragedSample(4);break;
         	case SEL_SINKTONE 	: gnSinkToneCps -= 50; if (gnSinkToneCps < SINKTONE_CPS_MAX) gnSinkToneCps = SINKTONE_CPS_MAX; nvd_WriteSinkToneCps(); nvd_UpdateFlash();break;
         	case SEL_CLIMBTONE 	: gnClimbToneCps -= 10; if (gnClimbToneCps < CLIMBTONE_CPS_MIN) gnClimbToneCps = CLIMBTONE_CPS_MIN; nvd_WriteClimbToneCps();nvd_UpdateFlash();break;
         	case SEL_FLIGHT_TRIG	: gnFlightTrigM--; if (gnFlightTrigM < FLIGHT_TRIG_MIN) gnFlightTrigM = FLIGHT_TRIG_MIN; nvd_WriteFlightTrig();nvd_UpdateFlash();break;
         	case SEL_RTC_HRS 	: gnRealHrs--; if (gnRealHrs == 255) gnRealHrs = 23; gnRealSec = 0; rtc_WriteSec();rtc_WriteMin();rtc_WriteHrs(); break;
         	case SEL_RTC_MIN 	: gnRealMin--; if (gnRealMin == 255) gnRealMin = 59; gnRealSec = 0; rtc_WriteSec();rtc_WriteMin(); break;
         	case SEL_RTC_DOM 	: gnDom--; if (gnDom < 1) gnDom = 31; rtc_WriteDom(); break;
         	case SEL_RTC_MON 	: gnMon--; if (gnMon < 1) gnMon = 12; rtc_WriteMon(); break;
         	case SEL_RTC_YAR 	: gnYar--; if (gnYar == 255) gnYar = 99; rtc_WriteYar(); break;
         	}
      	ui_DisplaySettings();
      	}
   	else
   	if (gbBtnRPressed || (gBtnRState == 0UL))  {
	  	ui_BtnClear();
      	switch (gnParSel) {
         	case SEL_ALTITUDE 	:  default : gnAltCorrM++; if (gnAltCorrM > ALT_CORR_MAX) gnAltCorrM = ALT_CORR_MAX; nvd_WriteAltCorr(); nvd_UpdateFlash();sns_AcquireAveragedSample(4);break;
         	case SEL_SINKTONE 	: gnSinkToneCps += 50; if (gnSinkToneCps > SINKTONE_CPS_MIN) gnSinkToneCps = SINKTONE_CPS_MIN; nvd_WriteSinkToneCps();nvd_UpdateFlash();break;
         	case SEL_CLIMBTONE 	: gnClimbToneCps += 10; if (gnClimbToneCps > CLIMBTONE_CPS_MAX) gnClimbToneCps = CLIMBTONE_CPS_MAX; nvd_WriteClimbToneCps();nvd_UpdateFlash();break;
         	case SEL_FLIGHT_TRIG	: gnFlightTrigM++; if (gnFlightTrigM > FLIGHT_TRIG_MAX) gnFlightTrigM = FLIGHT_TRIG_MAX; nvd_WriteFlightTrig();nvd_UpdateFlash();break;
         	case SEL_RTC_HRS 	: gnRealHrs++; if (gnRealHrs > 23) gnRealHrs = 0; gnRealSec = 0; rtc_WriteSec(); rtc_WriteMin(); rtc_WriteHrs(); break;
         	case SEL_RTC_MIN 	: gnRealMin++; if (gnRealMin > 59) gnRealMin = 0; gnRealSec = 0; rtc_WriteSec(); rtc_WriteMin();  break;
         	case SEL_RTC_DOM 	: gnDom++; if (gnDom > 31) gnDom = 1; rtc_WriteDom(); break;
         	case SEL_RTC_MON 	: gnMon++; if (gnMon > 12) gnMon = 1; rtc_WriteMon(); break;
         	case SEL_RTC_YAR 	: gnYar++; if (gnYar > 99) gnYar = 0; rtc_WriteYar(); break;
         	}
      	ui_DisplaySettings();
      	}
   	}
	}
   

void ui_FLogEventHandler(void)   {
   if (gbBtnLPressed)  {
	  ui_BtnClear();
      gnFLogRec--;
      if (gnFLogRec == 255) gnFLogRec = FLOG_MAX_FLIGHTS-1;
      ui_DisplayFLog();
      }
   else
   if (gbBtnRPressed)  {
	  ui_BtnClear();
      gnFLogRec++;
      if (gnFLogRec >= FLOG_MAX_FLIGHTS) gnFLogRec = 0;
      ui_DisplayFLog();
      }
   else
   if (gbBtnUPressed)  {
	  ui_BtnClear();
      tmr1_BusyWaitMs(200);
      if (gbBtnDPressed)  {
  	  	 ui_BtnClear();
      	 tmr1_BusyWaitMs(200);
   		 if (gbBtnUPressed)  {
	       	ui_BtnClear();
         	lcd_ClearScreen();
         	lcd_PrintSz(0,0,"ERASING LOG");
         	lcd_PrintSz(1,0,"...");
         	memset(gFlashBuf,0,NVD_PAR_OFFSET);
         	gnFLogOldestRec = 0;
         	nvd_WriteFLogOldestRec();
		 	nvd_UpdateFlash();
         	lcd_PrintSz(2,0,"DONE");
		 	tmr1_BusyWaitMs(2000);
         	lcd_ClearScreen();
         	gnFLogRec = 0;
         	ui_DisplayFLog();
         	}
		}
      }
   else
   if (gbBtnCPressed)  {
   	  ui_BtnClear();
      ui_InitSetMode();
      }
  }


void ui_RunEventHandler()  {
   sns_SampleContinuous();

/*
   if (gbBtnLPressed) {gbBtnLPressed = 0; gnCps -= 10;}
   else
   if (gbBtnRPressed) {gbBtnRPressed = 0; gnCps += 10;}
*/
   if ((!gbTimerEn) && gbStable && (ABS(((s16)gnAltAvgM) - gnLaunchAltM) > ((s16)gnFlightTrigM)))  {
      gbTimerEn = 1; // flight start
      gnAudioMode = AUDIO_MODE_SPEED;
      gnLaunchHrs = gnRealHrs;
      gnLaunchMin = gnRealMin;
      }
   if (gbBtnUPressed || gbBtnDPressed || gbBtnLPressed || gbBtnRPressed)  {
	  ui_BtnClear();
      if (!gbTimerEn) { // register launch altitude, zero flight time, turn off audio  
         gbStable = 1;
         gnMaxAltM = gnLZAltM = gnLaunchAltM = (s16)gnAltAvgM;
         gnMaxCps = 0;
         gnMinCps = 0;
         gnMaxTempC = gnMinTempC = gnTempC;
         gnTimerHrs = 0;
         gnTimerMin = 0;
		 gnAudioMode = AUDIO_MODE_OFF;
   		 audio_SetFreq(0); // shut off vario tone
         lcd_PrintSz(5,61,"RDY ");
         }
      }
    else 
    if (gbBtnCPressed)   {
	  	ui_BtnClear();
        gnAudioMode = AUDIO_MODE_OFF;
		audio_SetFreq(0); // shut off vario tone
       	gbStable = 0;
       	if (gbTimerEn) {// if flight started, this marks end of flight
          	gbTimerEn = 0;
          	gnLZAltM = (s16)gnAltAvgM;
          	ui_WriteFlightLog();
          	}
       	ui_InitFLogMode();
       	} 
   }

void ui_FlightScreenInit(void) {
	lcd_PrintLSz(0,45,"M");
	lcd_PrintSz(1,79,"$");
	lcd_PrintSz(3,30,"M");
	lcd_PrintSz(3,67,". ,");
	lcd_PrintSz(5,12,":   M");
	}

void ui_DisplaySettings(void)    {
    u8 n;
    s8 offset;
    offset = gnParSel - 5;
    if (offset < 0) offset = 0;
    for (n = 0; n < 6; n++)   {
       if (gnParSel == (n+offset)) lcd_PrintSz(n,0,"*");
       else lcd_PrintSz(n,0," ");
       }
    n = 0;
    if (offset < 1)
       {sprintf(gszBuf, "ALT%c%03d %04d",gnAltCorrM >= 0 ? '+' : '-',ABS(gnAltCorrM),(s16)gnAltAvgM);lcd_PrintSz(n,8,gszBuf); n++;}
    if (offset < 2)
       {sprintf(gszBuf, "SINK   -%d.%d",-gnSinkToneCps/100,((-gnSinkToneCps)%100)/10); lcd_PrintSz(n,8,gszBuf); n++;}
    if (offset < 3)
       {sprintf(gszBuf, "CLIMB  %c0.%02d",gnClimbToneCps < 0 ? '-' : '+',ABS(gnClimbToneCps));lcd_PrintSz(n,8,gszBuf); n++;}
    if (offset < 4)
       {sprintf(gszBuf, "FTRIG   %02d",gnFlightTrigM);lcd_PrintSz(n,8,gszBuf); n++;}
    if (offset < 5)
       {sprintf(gszBuf, "HOUR    %02u",gnRealHrs); lcd_PrintSz(n,8,gszBuf); n++;}
    if (offset < 6)
       {sprintf(gszBuf, "MINUTE  %02u",gnRealMin);lcd_PrintSz(n,8,gszBuf); n++;}
    if (offset < 7)
       {sprintf(gszBuf, "DATE    %02u",gnDom);lcd_PrintSz(n,8,gszBuf); n++;}
    if (offset < 8)
       {sprintf(gszBuf, "MONTH   %s",szMon[gnMon-1]);lcd_PrintSz(n,8,gszBuf); n++;}   
    if (offset < 9)
       {sprintf(gszBuf, "YEAR    %u",(u16)(2000+gnYar));lcd_PrintSz(n,8,gszBuf); n++;}
    }

void ui_InitRunMode(void)   {
   lcd_ClearScreen();
   sns_Init();
   sns_InitData();
   gnLaunchAltM = gnAltAvgM;
   ui_FlightScreenInit();
   gnAudioMode = AUDIO_MODE_SPEED;
   gnUIMode = UIMODE_RUN;
   }

void ui_InitSetMode(void)   {
   lcd_ClearScreen();
   gnParSel = 0;
   sns_AcquireAveragedSample(4);
   ui_DisplaySettings();
   gnUIMode = UIMODE_SET;
   }

void ui_InitFLogMode(void)   {
	lcd_ClearScreen();
	flash_Init();
   	gnFLogRec = 0;
  	ui_DisplayFLog();
   	gnUIMode = UIMODE_FLOG;
   	}


void ui_WriteFlightLog(void)   {
   gpFLogBuf = gFlashBuf + gnFLogOldestRec*FLOG_REC_BYTES;
   gpFLogBuf[0] = gnYar;
   gpFLogBuf[1] = gnMon;
   gpFLogBuf[2] = gnDom;
   gpFLogBuf[3] = gnLaunchHrs;   
   gpFLogBuf[4] = gnLaunchMin;
   gpFLogBuf[5] = gnTimerHrs;   
   gpFLogBuf[6] = gnTimerMin;
   gpFLogBuf[7] = (u8)(gnLaunchAltM & 0xFF);   
   gpFLogBuf[8] = (u8)((gnLaunchAltM>>8) & 0xFF);
   gpFLogBuf[9] = (u8)(gnMaxAltM & 0xFF);   
   gpFLogBuf[10] = (u8)((gnMaxAltM>>8) & 0xFF);   
   gpFLogBuf[11] = (u8)(gnLZAltM & 0xFF);   
   gpFLogBuf[12] = (u8)((gnLZAltM>>8) & 0xFF);
   gpFLogBuf[13] = (u8)(gnMaxCps/10);   
   gpFLogBuf[14] = (u8)(gnMinCps/10);
   gpFLogBuf[15] = (u8)gnMaxTempC;
   gpFLogBuf[16] = (u8)gnMinTempC;

   gnFLogOldestRec++;
   if (gnFLogOldestRec >= FLOG_MAX_FLIGHTS) {gnFLogOldestRec = 0;}
   nvd_WriteFLogOldestRec();
   nvd_UpdateFlash();
   }


// gnFLogOldestRec is the index of the oldest flight record in the circular buffer, the one
// which is free to write to
// gnFLogRec is the display logical index of the flight record, 0 being the most recent flight record, 
// FLOG_MAX_FLIGHTS-1 being the oldest flight record.
// ix is the corresponding physical index of the display record in the circular buffer

void ui_DisplayFLog(void)   {
	s8 ix;
	lcd_ClearScreen();
	sprintf(gszBuf,"     -%02u-",gnFLogRec+1);lcd_PrintSz(0,0,gszBuf);
	ix = gnFLogOldestRec - gnFLogRec - 1;
	// wrap around the circular buffer if needed to find the index of the record
	if (ix < 0) {ix += FLOG_MAX_FLIGHTS;}
	gpFLogBuf = gFlashBuf+ix*FLOG_REC_BYTES;
	CLAMP(gpFLogBuf[0],0,99);
	CLAMP(gpFLogBuf[1],1,12);
	CLAMP(gpFLogBuf[2],1,31);
	CLAMP(gpFLogBuf[3],0,23);
	CLAMP(gpFLogBuf[4],0,59);
	CLAMP(gpFLogBuf[5],0,9);
	CLAMP(gpFLogBuf[6],0,59);
	sprintf(gszBuf," 20%02u %s %02u",gpFLogBuf[0],szMon[gpFLogBuf[1]-1],gpFLogBuf[2]); lcd_PrintSz(1,0,gszBuf);
	sprintf(gszBuf,"%02u:%02u   %02u:%02u",gpFLogBuf[3],gpFLogBuf[4],gpFLogBuf[5],gpFLogBuf[6]); lcd_PrintSz(2,0,gszBuf);
	sprintf(gszBuf,"%04d +%02u.%u %d",(s16)gpFLogBuf[7] + (((s16)gpFLogBuf[8])<<8), gpFLogBuf[13]/10,gpFLogBuf[13]%10,(s8)gpFLogBuf[15]); lcd_PrintSz(3,0,gszBuf);
	sprintf(gszBuf,"%04d -%02u.%u %d",(s16)gpFLogBuf[9] + (((s16)gpFLogBuf[10])<<8),gpFLogBuf[14]/10,gpFLogBuf[14]%10,(s8)gpFLogBuf[16]) ; lcd_PrintSz(4,0,gszBuf);
	sprintf(gszBuf,"%04d",(s16)gpFLogBuf[11] + (((s16)gpFLogBuf[12])<<8) ); lcd_PrintSz(5,0,gszBuf);
	}

void ui_UpdateFlightData(void)  {
	if (((s16)gnAltAvgM) > gnMaxAltM) gnMaxAltM = (s16)gnAltAvgM;
	if (gnCps > gnMaxCps) gnMaxCps = gnCps;
	if (-gnCps > gnMinCps) gnMinCps = -gnCps;
	if (gnTempC > gnMaxTempC) gnMaxTempC = gnTempC;
	if (gnTempC < gnMinTempC) gnMinTempC = gnTempC;
	}

void ui_OpenScreen(void) {
	int volt,accum,cnt;
	lcd_ClearScreen();
	lcd_Printf(0,0," ALTI-VARIO");
	lcd_Printf(1,0,"   V 1.16");
    tmr1_BusyWaitMs(100);
    cnt = 8;
    accum = 0;
    while (cnt--) { 
    	volt = (int)adc_Convert();
		accum += volt;
		}
	accum >>= 3;
    volt = (int)(((long)accum * 6000L)>>10);
    volt = (volt+50)/100;
    lcd_Printf(3,0," BATTERY %d.%dV",volt/10,volt%10);
#ifdef UART_DBG
  	sprintf(gszBuf,"Bat Voltage %d/10 V\r\n",volt);
    uart_Print(gszBuf);
#endif
    tmr1_BusyWaitMs(2500); 
	}

void ui_BtnClear(void) {
	gbBtnLPressed = 0;
	gbBtnRPressed = 0;
	gbBtnUPressed = 0;
	gbBtnDPressed = 0;
	gbBtnCPressed = 0;
	}

